<?php
require_once('XmlTreeBuilder.php'); 
require_once('PathTool.php');	
require_once('ValidateData.php');
require_once('DTblDef.php');

class ConfManager
{
	var $_serv = array();
	var $_vhosts = array();
	var $_servErr = array();
	var $_vhostsErr = array();
	var $_modTime = array();

	var $_menu = array();
	var $_isAdm = false;
	var $_isChanged = false;

	function reset($type, $id)
	{ 
		if ( $type == 'vh' )
		{
			unset($this->_serv['vhTop'][$id]);
			unset($this->_servErr['vhTop'][$id]);
			unset($this->_modTime['vh'][$id]);
		}
	}

	function &exportConf()
	{
		$data = array();
		$listeners = array();
		$lnames = array_keys($this->_serv['listeners']);
		foreach( $lnames as $ln )
		{
			$l = &$this->_serv['listeners'][$ln];
			$ip = $l['ip'];
			if ( $ip == 'ANY' )
				$ip = '*';
			$addr = $ip . ':' . $l['port'];
			$listeners[$ln] = $addr;
		}
		$data['listener'] = $listeners;
		$data['vhost'] = array_keys($this->_serv['vhTop']);
		$data['serv'] = $this->_serv['general']['serverName'];
		return $data;
	}

	function &parseXml(&$filename)
	{
		if ( !is_file($filename) )
			return NULL;
		$fd = fopen ($filename, 'r');
		if ( !$fd )
			return NULL;
		
		$contents = fread ($fd, filesize ($filename)); 
		fclose ($fd); 
	
		$xmltree = new XmlTreeBuilder; 
		$root = &$xmltree->parse($contents); 
		return $root;
	}

	function checkModify($type, $id)
	{
		clearstatcache();
		if ( $type == 'confMgr' )
		{
			$curtime = filemtime($this->_modTime['serv']['file']);
			if ( $curtime > $this->_modTime['serv']['time'] )
			{
				$this->loadConfig('confMgr');
				return true;
			}
			return false;
		}
		elseif ( $type == 'admConfMgr' )
		{
			if ( $this->checkModify( 'confMgr', NULL) )
			{
				$root = $_SESSION['admRoot'] . "/conf/admin_config.xml" ;
				$this->_modTime['adm']['file'] = $root;
			}
			$curtime = filemtime($this->_modTime['adm']['file']);
			if ( $curtime > $this->_modTime['adm']['time'] )
			{
				$this->loadConfig('admConfMgr');
				return true;
			}
			return false;
		}
		elseif ( $type == 'vh' )
		{
			if ( file_exists($this->_modTime['vh'][$id]['file']) )
			{ 
				$curtime = filemtime($this->_modTime['vh'][$id]['file']);
				if ( $curtime > $this->_modTime['vh'][$id]['time'] )
				{
					$this->loadVhConfig($id);
					return true;
				}
				else
					return false;
			}
			else
			{
				$this->loadVhConfig($id);
				return true;
			}
		}
	}

	function loadConfig($id)
	{
		if ( $id == 'admConfMgr' )
			return $this->loadAdminConfig();

		$this->_isAdm = false;
		$root = $_SERVER['HTTPD_SERVER_ROOT'] . "conf/httpd_config.xml" ;
		$this->_modTime['serv']['file'] = $root;

		$rootNode = &$this->parseXml($root);
		if ( $rootNode != NULL )
		{
			$this->extractServConf($rootNode);
			$this->validateServ();
			$admRoot = $this->getAbsFile($this->_serv['general']['adminRoot'],
										 'SR', NULL);
			$_SESSION['admRoot'] = $admRoot;
			clearstatcache();
			$this->_modTime['serv']['time'] = filemtime($root);
			$_SESSION['confMgr'] = &$this;
			return true;
		}
		return false;
	}

	function loadAdminConfig()
	{
		$this->_isAdm = true;
		$admRoot = $_SESSION['admRoot'];
		if ( $admRoot == NULL )
		{
			$conf1 = new ConfManager();
			if ( !$conf1->loadConfig('confMgr') )
				return false;
			$admRoot = $this->getAbsFile($conf1->_serv['general']['adminRoot'],
										 'SR', NULL);
			$_SESSION['admRoot'] = $admRoot;
		}
		$root = $admRoot . "/conf/admin_config.xml" ;
		$this->_modTime['adm']['file'] = $root;
		$sroot = $_SERVER['HTTPD_SERVER_ROOT'] . "conf/httpd_config.xml" ;
		$this->_modTime['serv']['file'] = $sroot;

		$rootNode = &$this->parseXml($root);
		if ( $rootNode != NULL )
		{
			$this->extractAdminConf($rootNode);
			$this->validateAdmin();
			clearstatcache();
			$this->_modTime['serv']['time'] = filemtime($sroot);
			$this->_modTime['adm']['time'] = filemtime($root);
			$_SESSION['admConfMgr'] = &$this;
			return true;
		}
		return false;
	}

	function getServerLogFileName()
	{
		return $this->getAbsFile($this->_serv['general']['log_fileName'], 'SR',NULL);
	}

	function &loadMime()
	{
		$filename = $this->getAbsFile($this->_serv['general']['mime'], 'SR',NULL);
		$fd = fopen($filename, 'r');
		if ( !$fd ) 
			return false;
		$mime = array();
		while ( !feof($fd) )
		{
			$line = fgets($fd);
			$c = strpos($line, '=');
			if ( $c > 0 )
			{
				$suffix = trim(substr($line, 0, $c));
				$type = trim(substr($line, $c+1 ));
				$mime[$suffix] = $type;
			}
		}
		fclose($fd);
		ksort($mime);
		reset($mime);
		return $mime;
	}

	function saveMime(&$mime)
	{
		$filename = $this->getAbsFile($this->_serv['general']['mime'], 'SR', NULL);
		$fd = fopen($filename, 'w');
		if ( !$fd ) 
			return false;
		ksort($mime);
		reset($mime);
		foreach( $mime as $key => $value )
		{
			if ( strlen($key) < 8 )
				$key = substr($key . '        ', 0, 8);
			fputs( $fd, "$key = $value\n" );
		}
		fclose($fd);
		$this->_isChanged = true;
		return true;
	}

	function &loadUserDB($vhname, $realm)
	{
		$loc = $this->_vhosts[$vhname]['security']['realm'][$realm]['userDB/location'];

		$filename = $this->getAbsFile($loc, 'VR', $vhname);
		if ( PathTool::isDenied($filename) )
			return false;

		$lines = file($filename);
		if ( $lines == false ) 
			return false;
		$udb = array();

		foreach( $lines as $line )
		{
			rtrim($line);
			$c = strpos($line, ':');
			if ( $c > 0 )
			{
				$name = trim(substr($line, 0, $c));
				$passwd = trim(substr($line, $c+1 ));
				$c = strpos($passwd, ':');
				$group = '';
				if ( $c !== false )
				{
					$group = substr($passwd, $c+1);
					$passwd = substr($passwd, 0, $c);
				}
				$user = array();
				$user['passwd'] = $passwd;
				$user['group'] = $group;
				$udb[$name] = $user;
			}
		}
		ksort($udb);
		reset($udb);
		return $udb;
	}

	function saveUserDB(&$udb, $vhname, $realm)
	{
		$loc = $this->_vhosts[$vhname]['security']['realm'][$realm]['userDB/location'];

		$filename = $this->getAbsFile($loc, 'VR', $vhname);
		if ( PathTool::isDenied($filename) )
			return false;

		$fd = fopen($filename, 'w');
		if ( !$fd ) 
			return false;
		ksort($udb);
		reset($udb);
		foreach( $udb as $name => $user )
		{
			$pass = $user['passwd'];
			$grp = $user['group'];
			$line = "$name:$pass";
			if ( $grp != NULL )
				$line .= ":$grp";
			fputs( $fd, "$line\n" );
		}
		fclose($fd);
		return true;
	}

	function &loadGroupDB($vhname, $realm)
	{
		$loc = $this->_vhosts[$vhname]['security']['realm'][$realm]['groupDB/location'];
		$filename = $this->getAbsFile($loc, 'VR', $vhname);
		if ( PathTool::isDenied($filename) )
			return false;
		$lines = file($filename);
		if ( $lines == false ) 
			return false;
		$gdb = array();
		foreach( $lines as $line )
		{
			rtrim($line);
			$c = strpos($line, ':');
			if ( $c > 0 )
			{
				$name = trim(substr($line, 0, $c));
				$users = trim(substr($line, $c+1 ));
				$gdb[$name] = $users;
			}
		}
		ksort($gdb);
		reset($gdb);
		return $gdb;
	}

	function saveGroupDB(&$gdb, $vhname, $realm)
	{
		$loc = $this->_vhosts[$vhname]['security']['realm'][$realm]['groupDB/location'];
		$filename = $this->getAbsFile($loc, 'VR', $vhname);
		if ( PathTool::isDenied($filename) )
			return false;
		$fd = fopen($filename, 'w');
		if ( !$fd ) 
			return false;
		ksort($gdb);
		reset($gdb);
		foreach( $gdb as $name => $users )
		{
			$line = "$name:$users\n";
			fputs( $fd, $line );
		}
		fclose($fd);
		return true;
	}



	function loadVhConfig($vhname)
	{
		$vhRootNode = &$this->loadVhConfFile($vhname);
		if ( $vhRootNode != NULL )
		{
			$this->extractVhConf($vhname, $vhRootNode);
			$this->validateVhost($vhname);
			return true;
		}
		return false;
	}

	function &loadVhConfFile($vhname)
	{
		$vh = &$this->_serv['vhTop'][$vhname];
		$confFile = $this->getAbsFile($vh['configFile'], 'VR', $vhname);
		$this->_modTime['vh'][$vhname]['file'] = $confFile;

		$vhConfRoot = &$this->parseXml($confFile);
		if ( $vhConfRoot == NULL )
		{
			echo '<span class=body-error>ERROR: failed to parse xml' . $confFile . '</span>';
		}
		else
		{
			clearstatcache();
			$this->_modTime['vh'][$vhname]['time'] = filemtime($confFile);
		}

		return $vhConfRoot;
	}

	function extractServConf(&$rootNode)
	{
		$this->_serv = array();
		$this->_servErr = array();
		$this->_vhosts = array();
		$this->_vhostsErr = array();
		//unset( $this->_modTime );
		$el = &$rootNode['ELEMENTS'];
		$generalTags = array( 'serverName', 'adminEmails', 'mime', 'autoRestart',
							  'chrootPath', 'enableChroot', 'inMemBufSize',
							  'swappingDir', 'user', 'group', 'adminRoot',
							  'indexFiles', 'showVersionNumber', 'priority');
		for( $i = 0 ; $i < count($el) ; ++$i )
		{
			$name = $el[$i]['TAG'];
			$el1 = &$el[$i]['ELEMENTS'];

			if ( in_array($name, $generalTags) )
			{
				$this->_serv['general'][$name] = $el[$i]['VALUE'];
			}
			elseif ( $name == 'logging' )
			{
				$this->extractLogging($el1, $this->_serv['general']);
			}
			elseif ( $name == 'htAccess' )
			{
				$this->extractHtAccess($el1, $this->_serv['general']);
			}
			elseif ( $name == 'expires' )
			{
				$this->extractExpires($el1, $this->_serv['general']);
			}
			elseif ( $name == 'tuning' )
			{
				$this->_serv['tuning'] = &$this->extractServTuning($el1);
			}
			elseif ( $name == 'security' )
			{
				$this->_serv['security'] = &$this->extractServSecurity($el1);
			}
			elseif ( $name == 'virtualHostList' )
			{
				$this->extractVHosts($el1);
			}
			elseif ( $name == 'listenerList' )
			{
				$this->extractListeners($el1);
			}
			elseif ( $name == 'extProcessorList' )
			{
				$this->_serv['ext'] = &$this->extractExt($el[$i]['ELEMENTS']);
			}
			elseif ( $name == 'scriptHandlerList' )
			{
				$this->_serv['scriptHandler'] = &$this->extractScriptHandler( $el[$i]['ELEMENTS']);
				$keys = array_keys($this->_serv['scriptHandler']);
				foreach( $keys as $key )
				{
					$this->_serv['scriptHandler'][$key]['action'] = '<a href="javascript:wiz(\'ESscr\',\''.$key.'\',\'servScript1.php\')">Edit</a>&nbsp;&nbsp;&nbsp;<a href="javascript:del(\'DSscr\',\''.$key.'\')">Delete</a>';
				}
			}
		}
		$this->_serv['general']['runningAs'] = 'user('.$this->_serv['general']['user']. ') : group(' . $this->_serv['general']['group'] .')' ;
		$this->genMenuInfo();
	}

	function extractAdminConf(&$rootNode)
	{
		$this->_serv = array();
		$this->_servErr = array();
		//unset( $this->_modTime );
		$el = &$rootNode['ELEMENTS'];
		for( $i = 0 ; $i < count($el) ; ++$i )
		{
			$name = $el[$i]['TAG'];
			$el1 = &$el[$i]['ELEMENTS'];

			if ( $name == 'enableCoreDump' )
			{
				$this->_serv[$name] = $el[$i]['VALUE'];
			}
			elseif ( $name == 'logging' )
			{
				$this->extractLogging($el1, $this->_serv);
			}
			elseif ( $name == 'security' )
			{
				for ( $j = 0 ; $j < count($el1) ; ++$j )
				{
					if ( $el1[$j]['TAG'] == 'accessControl' )
						$this->extractAccessControl($el1[$j]['ELEMENTS'], $this->_serv);
				}
			}
			elseif ( $name == 'listenerList' )
			{
				$this->extractListeners($el1);
			}
		}
		$this->genMenuInfo();
	}

	function validateServ()
	{
		$serv = $this->_serv;
		$validate = new ValidateData();
		//general
		$tids = array('SERV_PROCESS', 'SERV_GENERAL', 'SERV_LOG', 'SERV_ACLOG', 
					  'A_HTACCESS', 'SERV_EXPIRES');
		$validate->extract( $tids, $this->_serv['general'], $serv['general'], 
							$this->_servErr['general']);
		//tuning
		$tids = array('SERV_TUNING_CONN', 'SERV_TUNING_REQ', 'SERV_TUNING_GZIP');
		$validate->extract( $tids, $this->_serv['tuning'], $serv['tuning'],
							$this->_servErr['tuning']); 

		//security 
		$tids = array('SERV_SEC_FILE', 'SERV_SEC_CGI','SERV_SEC_CONN','SERV_SEC_DENY', 'A_SECAC');
		$validate->extract( $tids, $this->_serv['security'], $serv['security'],
							$this->_servErr['security']); 

		// ext
		$tids = 'A_EXT';
		$exts = array_keys( $this->_serv['ext'] );
		$var_null = NULL;
		$tblDef = &$GLOBALS['_DTblDef_'];
		$table = &$tblDef->getTblDef($tids);
		$table->_dattrs[6]->_minVal = 2;
		foreach ( $exts as $ext )
		{
			$validate->extract( $tids, $this->_serv['ext'][$ext],
								$serv['ext'][$ext], $this->_servErr['ext'][$ext] );
			$validate->check( $tids, $this->_serv['ext'][$ext],
							$serv['ext'][$ext], $this->_servErr['ext'][$ext], $var_null );
		}
		$this->refreshExtApp(NULL, $serv);

		// listeners
		$tids = array( array('L_GENERAL', 'L_CERT'), 'L_VHMAP' );
		$lnames = array_keys( $this->_serv['listeners'] );
		foreach ( $lnames as $ln )
		{
			$validate->extract( $tids[0], $this->_serv['listeners'][$ln],
								$serv['listeners'][$ln], 
								$this->_servErr['listeners'][$ln] );
			if ( isset($this->_serv['listeners'][$ln]['vhmap']) ) 
			{
				$keys = array_keys( $this->_serv['listeners'][$ln]['vhmap'] );
				foreach( $keys as $key )
				{
					$validate->extract( $tids[1], $this->_serv['listeners'][$ln]['vhmap'][$key],
									$serv['listeners'][$ln]['vhmap'][$key], 
									$this->_servErr['listeners'][$ln]['vhmap'][$key] );
				}
			}
		}

		// vhTop
		$tids = 'VH_TOP';
		$vhnames = array_keys( $this->_serv['vhTop'] );
		foreach ( $vhnames as $vh )
		{
			$validate->extract( $tids, $this->_serv['vhTop'][$vh],
								$serv['vhTop'][$vh], $this->_servErr['vhTop'][$vh]);
		}

		// special validation
		$this->_serv = &$serv;
	}


	function validateAdmin()
	{
		$serv = $this->_serv;
		$validate = new ValidateData();
		$tblDef = &$GLOBALS['_DTblDef_'];
		$_SESSION['VH_ROOT'] = $_SESSION['admRoot'];
		//general
		$tids = array('ADMIN_PHP', 'VH_LOG', 'VH_ACLOG');
		$validate->extract( $tids, $this->_serv, $serv, $this->_servErr);

		//security
		$tids = 'A_SECAC';
		$validate->extract( $tids, $this->_serv, $serv,	$this->_servErr); 

		// listeners
		$tids = array('L_GENERAL', 'L_CERT');
		$lnames = array_keys( $this->_serv['listeners'] );
		foreach ( $lnames as $ln )
		{
			$validate->extract( $tids, $this->_serv['listeners'][$ln],
								$serv['listeners'][$ln], 
								$this->_servErr['listeners'][$ln] );
		}

		// special validation
		$this->_serv = &$serv;
	}

	function validateVhost($name)
	{
		$validate = new ValidateData();

		$vh = &$this->_vhosts[$name];
		$vh1 = $this->_vhosts[$name];
		$err = &$this->_vhostsErr[$name];

		$tblDef = &$GLOBALS['_DTblDef_'];
		$vhroot = $this->getAbsFile('$VH_ROOT/', 'VR', $name);
		$docroot = $this->getAbsFile('$DOC_ROOT/', 'DR', $name);
		$vh1['VH_ROOT'] = $vhroot;
		$vh1['DOC_ROOT'] = $docroot;
		$_SESSION['VH_ROOT'] = $vhroot;
		$_SESSION['DOC_ROOT'] = $docroot;

		// general
		$tids = array( 'VH_GENERAL', 'VH_LOG', 'VH_ACLOG', 'VH_INDXF', 'A_HTACCESS', 'VH_EXPIRES' );
		$validate->extract( $tids, $vh['general'], $vh1['general'], $err['general'] );

		//security
		if ( isset( $vh['security']['accessControl'] ) )
		{
			$tids = 'A_SECAC' ;
			$validate->extract( $tids, $vh['security']['accessControl'], 
								$vh1['security']['accessControl'], $err['security']['accessControl'] );
		}
		if ( isset( $vh['security']['general'] ) )
		{
			$tids = 'VH_SECG' ;
			$validate->extract( $tids, $vh['security']['general'], 
								$vh1['security']['general'], $err['security']['general'] );
		}
		if ( isset( $vh['security']['hotlink'] ))
		{
			$tids = 'VH_SECHL';
			$validate->extract( $tids, $vh['security']['hotlink'], 
								$vh1['security']['hotlink'], $err['security']['hotlink'] );
		}
		/*if ( isset( $vh['security']['realm'] ) )
		{
			$tids = 'VH_REALM' ;
			$keys = array_keys($vh['security']['realm']);
			foreach ( $keys as $key )
			{
				$validate->extract( $tids, $vh['security']['realm'][$key], 
									$vh1['security']['realm'][$key],
									$err['security']['realm'][$key]);
			}
		}*/
		$this->refreshVhRealms($name, $vh1);
		$this->refreshExtApp($name, $vh1);

		// ext
		if ( isset( $vh['ext'] ) )
		{
			$tids = 'A_EXT';
			$exts = array_keys( $vh['ext'] );
			$sext['names'] = array_keys($this->_serv['ext']);
			$table = &$tblDef->getTblDef($tids);
			$table->_dattrs[6]->_minVal = 3;
			foreach ( $exts as $ext )
			{
				$validate->extract( $tids, $vh['ext'][$ext],
									$vh1['ext'][$ext], $err['ext'][$ext] );
				$validate->check( $tids, $vh1['ext'][$ext], $err['ext'][$ext], $sext );
			}

		}

		// contexts
		if ( isset( $vh['context']['NULL'] ) )
		{
			$table = &$tblDef->getTblDef('VH_CTXG');
			$table->_dattrs['realm']->_maxVal = &$vh1['realms'];
 
			$urikeys = array_keys( $vh['context']['NULL'] );
			foreach ( $urikeys as $uri )
			{
				$table->extractValue($vh['context']['NULL'][$uri],
								$vh1['context']['NULL'][$uri], $err['context']['NULL'][$uri] );
			}
		}
		if ( isset( $vh['context']['webapp'] ) )
		{
			$table = &$tblDef->getTblDef('VH_CTXJ');
			$table->_dattrs['handler']->_maxVal = &$vh1['servletHandlers'];
			$table->_dattrs['realm']->_maxVal = &$vh1['realms'];
			$urikeys = array_keys( $vh['context']['webapp'] );
			foreach ( $urikeys as $uri )
			{
				$table->extractValue($vh['context']['webapp'][$uri],
								$vh1['context']['webapp'][$uri], $err['context']['webapp'][$uri] );
			}
		}
		if ( isset( $vh['context']['servlet'] ) )
		{
			$table = &$tblDef->getTblDef('VH_CTXS');
			$table->_dattrs['handler']->_maxVal = &$vh1['servletHandlers'];
			$table->_dattrs['realm']->_maxVal = &$vh1['realms'];
			$urikeys = array_keys( $vh['context']['servlet'] );
			foreach ( $urikeys as $uri )
			{
				$table->extractValue($vh['context']['servlet'][$uri],
								$vh1['context']['servlet'][$uri], $err['context']['servlet'][$uri] );
			}
		}
		if ( isset( $vh['context']['fcgi'] ) )
		{
			$table = &$tblDef->getTblDef('VH_CTXF');
			$table->_dattrs['handler']->_maxVal = &$vh1['fcgiHandlers'];
			$table->_dattrs['realm']->_maxVal = &$vh1['realms'];
			$urikeys = array_keys( $vh['context']['fcgi'] );
			foreach ( $urikeys as $uri )
			{
				$table->extractValue( $vh['context']['fcgi'][$uri],
								$vh1['context']['fcgi'][$uri], $err['context']['fcgi'][$uri] );
			}
		}
		if ( isset( $vh['context']['proxy'] ) )
		{
			$table = &$tblDef->getTblDef('VH_CTXP');
			$table->_dattrs['handler']->_maxVal = &$vh1['webServers'];
			$urikeys = array_keys( $vh['context']['proxy'] );
			foreach ( $urikeys as $uri )
			{
				$table->extractValue($vh['context']['proxy'][$uri],
								$vh1['context']['proxy'][$uri], $err['context']['proxy'][$uri] );
			}
		}
		if ( isset( $vh['context']['cgi'] ) )
		{
			$table = &$tblDef->getTblDef('VH_CTXC');
			$table->_dattrs['realm']->_maxVal = &$vh1['realms'];
			$urikeys = array_keys( $vh['context']['cgi'] );
			foreach ( $urikeys as $uri )
			{
				$table->extractValue( $vh['context']['cgi'][$uri],
								$vh1['context']['cgi'][$uri], $err['context']['cgi'][$uri] );
			}
		}
		if ( isset( $vh['context']['redirect'] ) )
		{
			$table = &$tblDef->getTblDef('VH_CTXR');
			$table->_dattrs['realm']->_maxVal = &$vh1['realms'];
			$urikeys = array_keys( $vh['context']['redirect'] );
			foreach ( $urikeys as $uri )
			{
				$table->extractValue( $vh['context']['redirect'][$uri],
								$vh1['context']['redirect'][$uri], $err['context']['redirect'][$uri] );
			}
		}

		/*/ script handler
		if ( isset( $vh['scriptHandler'] ) )
		{
			$tids = 'A_SCRIPT' ;
			$cur_table->_dattrs[2]->_maxVal = NULL;
			$validate->extract( $tids, $vh['scriptHandler'],
								$vh1['scriptHandler'], $err['scriptHandler'] );
		}*/

		$this->_vhosts[$name] = &$vh1;

	}

	function &extractServTuning(&$el)
	{
		$tuning = array();
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$name = $el[$i]['TAG'];
			$tunning[$name] = $el[$i]['VALUE'];
		}
		return $tunning;
	}

	function &extractScriptHandler(&$el)
	{
		$scriptHandler = array();
		$tags = array('suffix', 'type', 'handler');
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$el1 = &$el[$i]['ELEMENTS'];
			$sc = array();
			for ( $j = 0 ; $j < count($el1) ; ++ $j )
			{
				$name = $el1[$j]['TAG'];
				if ( in_array($name, $tags) )
					$sc[$name] = $el1[$j]['VALUE'];
			}
			if ( $sc['type'] && $sc['suffix'] )
			{
				$scriptHandler[$sc['suffix']] = &$sc;
			}
			unset( $sc );
		}
		return $scriptHandler;
	}

	function &extractExt(&$el)
	{
		$ext = array();
		$tags = array('address', 'name', 'type', 'maxConns', 'initTimeout',
					  'retryTimeout', 'autoStart', 'backlog', 'path', 'instances');
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$f = array();
			$el1 = &$el[$i]['ELEMENTS'];
			for ( $j = 0 ; $j < count($el1) ; ++$j )
			{
				$name = $el1[$j]['TAG'];
				if ( in_array($name, $tags) )
				{
					$f[$name] = $el1[$j]['VALUE'];
				}
				elseif ( $name == 'env' )
				{
					$f['env'][] = $el1[$j]['VALUE'];
				}
			}
			if ( isset($f['name']) )
				$ext[$f['name']] = &$f;
			unset($f);
		}
		return $ext;
	}

	function &extractServSecurity(&$el)
	{
		$sec = array();
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$name = $el[$i]['TAG'];
			if ( $name == 'fileAccessControl' )
			{
				$tags1 = array('followSymbolLink', 'checkSymbolLink', 
							  'requiredPermissionMask', 'restrictedPermissionMask' );
				$el1 = &$el[$i]['ELEMENTS'];
				for ( $j = 0 ; $j < count($el1) ; ++$j )
				{
					$name1 = $el1[$j]['TAG'];
					if ( in_array($name1, $tags1) )
					{
						$sec[$name.'/'.$name1] = $el1[$j]['VALUE'];
					}
				}
			}
			elseif ( $name == 'CGIRLimit' )
			{
				$el1 = &$el[$i]['ELEMENTS'];
				$tags1 = array('maxCGIInstances','minUID', 'minGID', 'priority', 
							   'CPUSoftLimit','CPUHardLimit','memSoftLimit',
							   'memHardLimit','procSoftLimit','procHardLimit');
				for ( $j = 0 ; $j < count($el1) ; ++$j )
				{
					$name1 = $el1[$j]['TAG'];
					if ( in_array($name1, $tags1) )
					{
						$sec[$name.'/'.$name1] = $el1[$j]['VALUE'];
					}
				}
			}
			elseif ( $name == 'perClientConnLimit' )
			{
				$el1 = &$el[$i]['ELEMENTS'];
				$tags1 = array('throttleLimit','softLimit','hardLimit','gracePeriod','banPeriod');
				for ( $j = 0 ; $j < count($el1) ; ++$j )
				{
					$name1 = $el1[$j]['TAG'];
					if ( in_array($name1, $tags1) )
					{
						$sec[$name.'/'.$name1] = $el1[$j]['VALUE'];
					}
				}
			}
			elseif ( $name == 'accessDenyDir' )
			{
				$el1 = &$el[$i]['ELEMENTS'];
				for ( $j = 0 ; $j < count($el1) ; ++$j )
				{
					if ( $el1[$j]['TAG'] == 'dir' )
						$sec[$name][] = $el1[$j]['VALUE'];
				}
			}
			elseif ( $name == 'accessControl' )
			{
				$this->extractAccessControl($el[$i]['ELEMENTS'], $sec);
			}
		}
		return $sec;
	}

	function extractAccessControl(&$el, &$sec)
	{
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$name = $el[$i]['TAG'];
			if ( $name == 'allow' )
				$sec['accessControl_allow'] = $el[$i]['VALUE'];
			elseif ( $name == 'deny' )
				$sec['accessControl_deny'] = $el[$i]['VALUE'];
		}
	}

	function &extractHtAccess(&$el, &$holder)
	{
		$tags = array('allowOverride', 'accessFileName');
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$name = $el[$i]['TAG'];
			$value = $el[$i]['VALUE'];
			if ( in_array($name, $tags) )
				$holder[$name] = $value;
		}
	}

	function &extractExpires(&$el, &$holder)
	{
		$tags = array('enableExpires', 'expiresByType', 'expiresDefault');
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$name = $el[$i]['TAG'];
			$value = $el[$i]['VALUE'];
			if ( in_array($name, $tags) )
				$holder[$name] = $value;
		}
	}


	function &extractLogging(&$el, &$holder)
	{
		$logTags = array('fileName', 'useServer', 'logLevel',
						 'debugLevel', 'rollingSize', 'enableStderrLog');
		$aclogTags = array('fileName', 'useServer', 'keepDays',
						   'rollingSize', 'logReferer', 'logUserAgent');
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$name = $el[$i]['TAG'];
			if ( $name == 'log' )
			{
				$el1 = &$el[$i]['ELEMENTS'];
				for ( $j = 0 ; $j < count($el1) ; ++$j )
				{
					$name = $el1[$j]['TAG'];
					if ( in_array($name, $logTags) )
						$holder["log_$name"] = $el1[$j]['VALUE'];
				}
			}
			elseif ( $name == 'accessLog' )
			{
				$el1 = &$el[$i]['ELEMENTS'];
				for ( $j = 0 ; $j < count($el1) ; ++$j )
				{
					$name = $el1[$j]['TAG'];
					if ( in_array($name, $aclogTags) )
						$holder["accessLog_$name"] = $el1[$j]['VALUE'];
				}
			}
		}
	}

	function extractListeners(&$el)
	{
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$l = array();

			$el1 = &$el[$i]['ELEMENTS'];
			for ( $j = 0 ; $j < count($el1) ; ++$j )
			{
				$name = $el1[$j]['TAG'];
				if ( $name == 'vhostMapList' )
				{
					$el2 = &$el1[$j]['ELEMENTS'];
					for ( $k = 0 ; $k < count($el2) ; ++$k )
					{
						$el3 = $el2[$k]['ELEMENTS'];
						for ( $a = 0 ; $a < count($el3) ; ++$a )
						{
							$name2 = $el3[$a]['TAG'];
							if ( $name2 == 'vhost' )
							{
								$vh = $el3[$a]['VALUE'];
							}
							elseif ( $name2 == 'domain' )
								$domain = $el3[$a]['VALUE'];
						}
						if ( isset($vh) && isset($domain) )
						{
							$l['vhmap'][$vh]['vh'] = $vh;
							$l['vhmap'][$vh]['domain'] = $domain;
						}
					}
				}
				else
					$l[$name] = $el1[$j]['VALUE'];

				if ( $name == 'address' )
				{
					$addr = $el1[$j]['VALUE'];
					$pos = strpos($addr,':');
					if ( $pos )
					{
						$l['ip'] = substr($addr, 0, $pos);
						$l['port'] = substr($addr, $pos+1);
						if ( $l['ip'] == '*' )
							$l['ip'] = 'ANY';
					}
				}
			}
			if ( isset($l['name']) )
			{
				$l['action'] = '<a href="javascript:wiz(\'ELgeneral\', \''.$l['name'].'\',\'listenerGeneral1.php\')">Edit</a>&nbsp;&nbsp;&nbsp;<a href="javascript:del(\'DL\', \''.$l['name'].'\')">Delete</a>';
				$this->_serv['listeners'][$l['name']] = &$l ;
			}

			unset($l);
		}
	}

	function extractVHosts(&$el)
	{
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$vh = array();

			$el1 = &$el[$i]['ELEMENTS'];
			for ( $j = 0 ; $j < count($el1) ; ++$j )
			{
				$name = $el1[$j]['TAG'];
				if ( $name == 'name' )
					$vname = $el1[$j]['VALUE'];
				$vh[$name] = $el1[$j]['VALUE'];
			}
			if ( isset($vname) )
				$this->_serv['vhTop'][$vname] = &$vh ;
			unset($vh);
		}
	}

	function extractVhConf($vhname, $vhRootNode)
	{
		$el = &$vhRootNode['ELEMENTS'];
		if ( !isset($el) )
			return false;
		
		unset($this->_vhosts[$vhname]);
		unset($this->_vhostsErr[$vhname]);
		$vh = &$this->_vhosts[$vhname];
		$generalTags = array('docRoot', 'enableGzip', 'adminEmails', 'indexFiles', 'indexFiles_useServer');
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$name = $el[$i]['TAG'];
			$el1 = &$el[$i]['ELEMENTS'];
			if ( in_array($name, $generalTags) )
			{
				$vh['general'][$name] = $el[$i]['VALUE'];
			}
			elseif ( $name == 'contextList' )		
			{
				$vh['context'] = &$this->extractContextList($el1);
			}
			elseif ( $name == 'scriptHandlerList' )
			{
				$vh['scriptHandler'] = &$this->extractScriptHandler($el1);
				$keys = array_keys($vh['scriptHandler']);
				foreach( $keys as $key )
				{
					$vh['scriptHandler'][$key]['action'] = '<a href="javascript:wiz(\'EVscr'.$key."','".$vhname.'\',\'vhScript1.php\')">Edit</a>&nbsp;&nbsp;&nbsp;<a href="javascript:del(\'DVscr'.$key."','".$vhname.'\')">Delete</a>';
				}
			}
			elseif ( $name == 'customErrorPages' )
			{
				$vh['errUrl'] = array();
				for ( $j = 0 ; $j < count($el1) ; ++$j )
				{
					$errUrl = array();
					$el2 = &$el1[$j]['ELEMENTS'];
					for ( $k = 0 ; $k < count($el2) ; ++ $k )
					{
						$name1 = $el2[$k]['TAG'];
						if ( $name1 == 'errCode' )
							$errUrl['errCode'] = $el2[$k]['VALUE'];
						elseif ( $name1 == 'url' )
							$errUrl['url'] = $el2[$k]['VALUE'];
					}
					if ( isset($errUrl['errCode']) && isset($errUrl['url']) )
					{
						$errUrl['action'] = '<a href="javascript:wiz(\'EVerr'.$errUrl['errCode']."','".$vhname.'\',\'vhError1.php\')">Edit</a>&nbsp;&nbsp;&nbsp;<a href="javascript:del(\'DVerr'.$errUrl['errCode']."','".$vhname.'\')">Delete</a>';
						$vh['errUrl'][$errUrl['errCode']] = &$errUrl;
					}
					unset( $errUrl );
				}
			}
			elseif ( $name == 'security' )		
			{
				$vh[$name] = &$this->extractVhSecurity($el1, $vhname);
			}
			elseif ( $name == 'logging' )
			{
				$this->extractLogging($el1, $vh['general']);
			}
			elseif ( $name == 'htAccess' )
			{
				$this->extractHtAccess($el1, $vh['general']);
			}
			elseif ( $name == 'expires' )
			{
				$this->extractExpires($el1, $vh['general']);
			}
			elseif ( $name == 'extProcessorList' )
			{
				$vh['ext'] = &$this->extractExt($el1);
			}
		}

		return true;

	}

	function &extractRealm(&$el1)
	{
		$realms = array();
		for( $i = 0 ; $i < count($el1) ; ++$i )
		{
			$el2 = $el1[$i]['ELEMENTS'];
			$realm = array();
			for ( $j = 0 ; $j < count($el2) ; ++$j )
			{
				$name = $el2[$j]['TAG'];
				if ( $name == 'name' )
					$realm['name'] = $el2[$j]['VALUE'];
				elseif ( $name == 'type' )
					$realm['type'] = $el2[$j]['VALUE'];
				elseif ( $name == 'userDB' )
				{
					$el3 = $el2[$j]['ELEMENTS'];
					for ( $k = 0 ; $k < count($el3) ; ++$k )
					{
						$name = $el3[$k]['TAG'];
						$val = $el3[$k]['VALUE'];
						if ( $name == 'location' )
							$realm['userDB/location'] = $val;
						elseif( $name == 'maxCacheSize' )
							$realm['userDB/maxCacheSize'] = $val;
						elseif ( $name == 'cacheTimeout' )
							$realm['userDB/cacheTimeout'] = $val;
					}
				}
				elseif ( $name == 'groupDB' )
				{
					$el4 = $el2[$j]['ELEMENTS'];
					for ( $l = 0 ; $l < count($el4); ++$l )
					{
						$name = $el4[$l]['TAG'];
						$val = $el4[$l]['VALUE'];
						if ( $name == 'location' )
							$realm['groupDB/location'] = $val;
						elseif( $name == 'maxCacheSize' )
							$realm['groupDB/maxCacheSize'] = $val;
						elseif ( $name == 'cacheTimeout' )
							$realm['groupDB/cacheTimeout'] = $val;
					}
				}
			}
			if ( isset($realm['name']) )
			{
				$realms[$realm['name']] = $realm;
			}
			unset( $realm );
		}
		return $realms;

	}

	function &extractHotlink(&$el1)
	{
		$hotlink = array();
		$tags = array('enableHotlinkCtrl', 'suffixes', 'allowDirectAccess', 
					  'redirectUri', 'onlySelf', 'allowedHosts');
		for ( $i = 0 ; $i < count($el1) ; ++$i )
		{
			$name = $el1[$i]['TAG'];
			if ( in_array($name, $tags) )
				$hotlink[$name] = $el1[$i]['VALUE'];
		}
		return $hotlink;
	}

	function &extractVhSecurity(&$el, $vhname)
	{
		$vhSec = array();
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$name = $el[$i]['TAG'];
			$el1 = $el[$i]['ELEMENTS'];
			if ( $name == 'accessControl' )
			{
				$vhSec['accessControl'] = array();
				$this->extractAccessControl($el1, $vhSec['accessControl']);
			}
			elseif ( $name == 'realmList' )
			{
				
				$realms = &$this->extractRealm($el1);
				$vhSec['realm'] = &$realms;
			}
			elseif ( $name == 'general' )
			{
				$tags = array('enableContextAC');
				for ( $j = 0 ; $j < count($el1) ; ++$j )
				{
					$name = $el1[$j]['TAG'];
					if ( in_array($name, $tags) )
						$vhSec['general'][$name] = $el1[$j]['VALUE'];
				}
			}
			elseif ( $name == 'hotlinkCtrl' )
			{
				$hotlink = &$this->extractHotlink( $el1 );
				$vhSec['hotlink'] = &$hotlink;
			}
		}
		return $vhSec;
	}

	function &extractContextList(&$el)
	{
		$ct = array();
		$tags = array('uri', 'location', 'type', 'handler', 'allowBrowse', 
					  'enableExpires', 'expiresDefault', 'allowOverride', 
					  'realm', 'authName', 'required', 'externalRedirect',
					  'statusCode', 'allowSetUID', 'addDefaultCharset',
					  'defaultCharsetCustomized', 'addMIMEType', 'forceType',
					  'defaultType');
		for ( $i = 0 ; $i < count($el) ; ++$i )
		{
			$el1 = &$el[$i]['ELEMENTS'];
			$ctx = array();
			for ( $j = 0 ; $j < count($el1) ; ++$j )
			{
				$name = $el1[$j]['TAG'];
				if ( in_array($name, $tags) )
					$ctx[$name] = $el1[$j]['VALUE'];
				elseif ( $name == 'accessControl' )
				{
					$this->extractAccessControl($el1[$j]['ELEMENTS'], $ctx);
				}
			}
			if ( $ctx['uri'] != NULL )
				$ct[$ctx['type']][$ctx['uri']] = $ctx;
			unset( $ctx );
		}
		return $ct;
	}

	function &createMenuItem($id, $type, $disp, $uri, $ico)
	{
		$mi = array();
		$mi['type'] = $type;
		$mi['disp'] = $disp;
		if ( $uri != NULL )
		{
			if ( $id != NULL )
				$uri .= "?id=$id";
			$mi['uri'] = $uri;
		}
		if ( $ico != NULL )
		{
			$mi['ico'] = $ico;
		}
		return $mi;
	}

	function genMenuInfo()
	{
		$this->_menu = array();
		$id = 0;

		if ( $this->_isAdm )
		{
			$name = 'Admin Server';
			$ltag = 'Admin Listeners';
			$servGeneral = 'adminServGeneral.php';
			$listenerTop = 'adminListenerTop.php';
			$listenerGeneral = 'adminListenerGeneral.php';
		}
		else
		{
			$name = 'Server';
			$ltag = 'Listeners';
			$servGeneral = 'servGeneral.php';
			$listenerTop = 'listenerTop.php';
			$listenerGeneral = 'listenerGeneral.php';
		}
		$this->_menu[$id] = &$this->createMenuItem($id, 'mh', $name, $servGeneral, NULL);
		++$id;

		$this->_menu[$id] = &$this->createMenuItem($id, 'mh', $ltag, $listenerTop, NULL);
		++$id;

		$keys = array_keys($this->_serv['listeners']);
		sort($keys);
		foreach( $keys as $name )
		{
			$ico = NULL;
			if ( $this->_serv['listeners'][$name]['secure'] == 1 )
				$ico = '<img src="../../images/graphics/lock.gif" width="12" height="15" alt="SSL Listener">';
			$this->_menu[$id] = &$this->createMenuItem($id, 'mi', $name, $listenerGeneral, $ico);
			++$id;
		}

		if ( !$this->_isAdm )
		{
			$this->_menu[$id] = &$this->createMenuItem($id, 'mh', 'Virtual Hosts', 'vhTop.php', NULL);
			++$id;

			$keys = array_keys($this->_serv['vhTop']);
			sort($keys);
			foreach( $keys as $name)
			{ 
				$this->_menu[$id] = &$this->createMenuItem($id, 'mi', $name, 'vhGeneral.php', NULL);
				++$id;
			}
		}

	}
	

	/*function dbg_out($tag, &$obj)
	{
		echo "<!-- $tag --\n";
		var_dump($obj);
		echo "-->\n";
	}
	function dbg_tag($tag)
	{
		echo "<!-- $tag -->\n";
	}*/

	function printRoot()
	{
		if ( isset($this->_rootNode) )
		{
			$xml2a = new xml2array;		
			$out = $xml2a->xmlNodeOutput( $this->_rootNode, 0);
			print $out;
			return true;
		}
		return false;
	}

	function del($tag, $objId)
	{
		if ( $tag == 'DSvh' )
		{
			unset( $this->_vhosts[$objId] );
			unset( $this->_vhostsErr[$objId] );
			unset( $this->_serv['vhTop'][$objId] );
			unset( $this->_servErr['vhTop'][$objId]);
			unset( $this->_modTime['vh'][$objId]);
			$this->saveServFile(true);
		}
		elseif ( $tag == 'DL' )
		{
			unset( $this->_serv['listeners'][$objId] );
			unset( $this->_servErr['listeners'][$objId] );
			$this->saveServFile(true);
		}
		elseif ( $tag == 'DSext' )
		{
			unset( $this->_serv['ext'][$objId] );
			unset( $this->_servErr['ext'][$objId] );
			$this->refreshExtApp( NULL, $this->_serv );
			$this->saveServFile(false);
		}
		elseif ( $tag == 'DSscr' )
		{
			unset( $this->_serv['scriptHandler'][$objId] );
			unset( $this->_servErr['scriptHandler'][$objId] );
			$this->saveServFile(false);
		}
		elseif ( $tag{1} == 'V' )
		{
			$delId = substr($tag, 5);
			$tag = substr($tag, 2, 3);
			if ( $tag == 'scr' )
			{
				unset( $this->_vhosts[$objId]['scriptHandler'][$delId] );
				unset( $this->_vhostsErr[$objId]['scriptHandler'][$delId] );
			}
			elseif ( $tag == 'err' )
			{
				unset( $this->_vhosts[$objId]['errUrl'][$delId] );
				unset( $this->_vhostsErr[$objId]['errUrl'][$delId] );
			}
			elseif ( $tag == 'hta' )
			{
				unset( $this->_vhosts[$objId]['security']['realm'][$delId] );
				unset( $this->_vhostsErr[$objId]['security']['realm'][$delId] );
				$this->refreshVhRealms( $objId, $this->_vhosts[$objId] );
			}
			elseif ( $tag =='ext' )
			{
				unset( $this->_vhosts[$objId]['ext'][$delId] );
				unset( $this->_vhostsErr[$objId]['ext'][$delId] );
				$this->refreshExtApp($objId, $this->_vhosts[$objId] );
			}
			elseif ( $tag == 'ctx' )
			{
				$deluri = substr($delId, 1);
				$type = $delId{0};
				$types = array('g'=>'NULL', 'j'=>'webapp', 's'=>'servlet',
							   'f'=>'fcgi', 'p'=>'proxy', 'c'=>'cgi', 'r'=>'redirect');
				$type = $types[$type];
				unset($this->_vhosts[$objId]['context'][$type][$deluri]);
				unset($this->_vhostsErr[$objId]['context'][$type][$deluri]);
			}
			$this->saveVhFile($objId);
					 
		}
	}

	function getAbsFile($filename, $type, $vhname)
	{
		// type = 'SR', 'VR', 'DR'
		$s = $filename{0};
		if ( $s == '$' )
		{
			if ( strncasecmp('$SERVER_ROOT', $filename, 12) == 0 )
			{
				$filename = $_SERVER['HTTPD_SERVER_ROOT'] . substr($filename, 13);
			}
			elseif ( ($type == 'VR' || $type == 'DR') 
					 && $vhname && (strncasecmp('$VH_ROOT', $filename, 8) == 0) )
			{
				$vhroot = $this->getAbsFile($this->_serv['vhTop'][$vhname]['vhRoot'],'SR',NULL);
				$filename = $vhroot . substr($filename, 9);			    
			}
			elseif ( $type == 'DR' && $vhname 
					 && (strncasecmp('$DOC_ROOT', $filename, 9) == 0) )
			{
				$docroot = $this->getAbsFile($this->_vhosts[$vhname]['general']['docRoot'], 'VR', $vhname);
				$filename = $docroot . substr($filename, 10);
			}

		}
		elseif ( $s == '/' )
		{
			if ( isset( $_SERVER['LSWS_CHROOT'] ) )
			{
				$root = $_SERVER['LSWS_CHROOT'];
				$len = strlen($root);
				if ( strncmp( $filename, $root, $len ) == 0 )
					$filename = substr($filename, $len);
			}
		}
		return $filename;

	}

	function saveVhFile($vhname)
	{
		$g1 = &$this->_serv['vhTop'][$vhname];
		$confFile = $this->getAbsFile($g1['configFile'], 'VR', $vhname);
		$fd = fopen($confFile, 'w');
		if ( !$fd )
			return false;

		fputs( $fd, '<?xml version="1.0" encoding="UTF-8"?>'."\n");
		$level = 0;
		fputs( $fd, $this->xmlTagS( $level, 'virtualHostConfig' ));

		$vh = &$this->_vhosts[$vhname];
		//general
		$g = &$vh['general'];
		////docRoot
		fputs( $fd, $this->xmlTag( $level, 'docRoot', $g['docRoot']));
		fputs( $fd, $this->xmlTag( $level, 'enableGzip', $g['enableGzip']));
		fputs( $fd, $this->xmlTag( $level, 'adminEmails', $g['adminEmails']));

		////logging
		fputs( $fd, $this->xmlTagS( $level, 'logging' ));
		if ( isset( $g['log_fileName'] ))
		{
			fputs( $fd, $this->xmlTagS($level, 'log' ));
			fputs( $fd, $this->xmlTag( $level, 'useServer', $g['log_useServer'] ));
			fputs( $fd, $this->xmlTag( $level, 'fileName', $g['log_fileName'] ));
			fputs( $fd, $this->xmlTag( $level, 'logLevel', $g['log_logLevel'] ));
			fputs( $fd, $this->xmlTag( $level, 'rollingSize', $g['log_rollingSize']));
			fputs( $fd, $this->xmlTagE( $level, 'log' ));
		}
		if ( isset( $g['accessLog_fileName'] ))
		{
			fputs($fd, $this->xmlTagS($level, 'accessLog' ));
			fputs( $fd, $this->xmlTag($level, 'useServer', $g['accessLog_useServer']));
			fputs( $fd, $this->xmlTag($level, 'fileName', $g['accessLog_fileName']));
			fputs( $fd, $this->xmlTag($level, 'logReferer', $g['accessLog_logReferer']));
			fputs( $fd, $this->xmlTag($level, 'logUserAgent', $g['accessLog_logUserAgent']));
			fputs( $fd, $this->xmlTag($level, 'rollingSize', $g['accessLog_rollingSize']));
			fputs( $fd, $this->xmlTag($level, 'keepDays', $g['accessLog_keepDays'] ));
			fputs($fd, $this->xmlTagE($level, 'accessLog' ));
		}
		fputs( $fd, $this->xmlTagE( $level, 'logging' ));

		////index files
		if ( isset( $g['indexFiles'] ))
			fputs( $fd, $this->xmlTag( $level, 'indexFiles', $g['indexFiles'] ));
		fputs( $fd, $this->xmlTag( $level, 'indexFiles_useServer', $g['indexFiles_useServer'] ));

		//htAccess
		if ( isset($g['allowOverride']) || isset($g['accessFileName']) )
		{ 
			fputs( $fd, $this->xmlTagS( $level, 'htAccess' ));
			fputs( $fd, $this->xmlTag( $level, 'allowOverride', $g['allowOverride'] ));
			fputs( $fd, $this->xmlTag( $level, 'accessFileName', $g['accessFileName'] ));
			fputs( $fd, $this->xmlTagE( $level, 'htAccess' ));
		}

		//expires
		fputs( $fd, $this->xmlTagS( $level, 'expires' ));
		fputs( $fd, $this->xmlTag(  $level, 'enableExpires', $g['enableExpires'] ));
		fputs( $fd, $this->xmlTag(  $level, 'expiresDefault', $g['expiresDefault'] ));
		fputs( $fd, $this->xmlTagE( $level, 'expires' ));

		//security
		if ( isset( $vh['security'] ))
		{
			$s = &$vh['security'];
			fputs( $fd, $this->xmlTagS( $level, 'security' ));
			fputs( $fd, $this->xmlTagS( $level, 'general' ));
			fputs($fd, $this->xmlTag( $level, 'enableContextAC', $s['general']['enableContextAC'] ));
			fputs( $fd, $this->xmlTagE( $level, 'general' ));
			//// hotlink
			fputs( $fd, $this->xmlTagS( $level, 'hotlinkCtrl' ));
			fputs( $fd, $this->xmlTag( $level, 'enableHotlinkCtrl', $s['hotlink']['enableHotlinkCtrl'] ));
			fputs( $fd, $this->xmlTag( $level, 'suffixes', $s['hotlink']['suffixes'] ));
			fputs( $fd, $this->xmlTag( $level, 'allowDirectAccess', $s['hotlink']['allowDirectAccess'] ));
			fputs( $fd, $this->xmlTag( $level, 'redirectUri', $s['hotlink']['redirectUri'] ));
			fputs( $fd, $this->xmlTag( $level, 'onlySelf', $s['hotlink']['onlySelf'] ));
			fputs( $fd, $this->xmlTag( $level, 'allowedHosts', $s['hotlink']['allowedHosts'] ));
			fputs( $fd, $this->xmlTagE( $level, 'hotlinkCtrl' ));
			
			//// realmList
			if ( isset($s['realm']) && count($s['realm']) > 0 )
			{
				fputs( $fd, $this->xmlTagS( $level, 'realmList' ));
				$keys = array_keys($s['realm']);
				sort( $keys );
				foreach ( $keys as $key )
				{
					$name = $key;
					$realm = $s['realm'][$key];
					fputs($fd, $this->xmlTagS($level, 'realm'));
					fputs($fd, $this->xmlTag( $level, 'name', $name ));
					fputs($fd, $this->xmlTag( $level, 'type', $realm['type']));
					fputs($fd, $this->xmlTagS($level, 'userDB'));
					fputs($fd, $this->xmlTag( $level, 'location', $realm['userDB/location']));
					fputs($fd, $this->xmlTag( $level, 'maxCacheSize', $realm['userDB/maxCacheSize']));
					fputs($fd, $this->xmlTag( $level, 'cacheTimeout', $realm['userDB/cacheTimeout']));
					fputs($fd, $this->xmlTagE($level, 'userDB'));
					fputs($fd, $this->xmlTagS($level, 'groupDB'));
					fputs($fd, $this->xmlTag( $level, 'location', $realm['groupDB/location']));
					fputs($fd, $this->xmlTag( $level, 'maxCacheSize', $realm['groupDB/maxCacheSize']));
					fputs($fd, $this->xmlTag( $level, 'cacheTimeout', $realm['groupDB/cacheTimeout']));
					fputs($fd, $this->xmlTagE($level, 'groupDB'));
					fputs($fd, $this->xmlTagE($level, 'realm'));
				}
				fputs( $fd, $this->xmlTagE( $level, 'realmList' ));
			}
		
		//// access control
			if ( $s['accessControl']['accessControl_allow'] != NULL 
				|| $s['accessControl']['accessControl_deny'] != NULL  )
			{
				fputs( $fd, $this->xmlTagS($level, 'accessControl' ));
				fputs( $fd, $this->xmlTag( $level, 'allow', $s['accessControl']['accessControl_allow']));
				fputs( $fd, $this->xmlTag( $level, 'deny', $s['accessControl']['accessControl_deny']));
				fputs( $fd, $this->xmlTagE($level, 'accessControl' ));
			}
			fputs( $fd, $this->xmlTagE( $level, 'security' ));
		}

		//ext
		if ( isset($vh['ext']) && count($vh['ext']) > 0 )
		{
			fputs( $fd, $this->xmlTagS( $level, 'extProcessorList' ));
			$ext0 = &$vh['ext'];
			foreach( $ext0 as $a )
			{
				$id = $a['type'] . $a['name'];
				$ext[$id] = $a;
			}
			ksort($ext);
			reset($ext);

			foreach( $ext as $f )
			{
				fputs( $fd, $this->xmlTagS( $level, 'extProcessor' ));
				fputs( $fd, $this->xmlTag(  $level, 'name', $f['name'] ));
				fputs( $fd, $this->xmlTag(  $level, 'address', $f['address'] ));
				fputs( $fd, $this->xmlTag(  $level, 'type', $f['type'] ));
				fputs( $fd, $this->xmlTag(  $level, 'maxConns', $f['maxConns'] ));
				fputs( $fd, $this->xmlTag(  $level, 'initTimeout', $f['initTimeout'] ));
				fputs( $fd, $this->xmlTag(  $level, 'retryTimeout', $f['retryTimeout'] ));
				if ( isset( $f['env'] ) )
				{
					foreach( $f['env'] as $value )
					{
						fputs( $fd, $this->xmlTag(  $level, 'env', $value ));
					}
				}
				if ( $f['type'] == 'fcgi' )
				{
					fputs( $fd, $this->xmlTag(  $level, 'autoStart', $f['autoStart'] ));
					fputs( $fd, $this->xmlTag(  $level, 'path', $f['path'] ));
					fputs( $fd, $this->xmlTag(  $level, 'backlog', $f['backlog'] ));
					fputs( $fd, $this->xmlTag(  $level, 'instances', $f['instances'] ));
				}
				fputs( $fd, $this->xmlTagE( $level, 'extProcessor' ));
			}
			fputs( $fd, $this->xmlTagE( $level, 'extProcessorList' ));
		}

		//contexts
		if ( isset($vh['context']) && count($vh['context']) > 0 )
		{
			fputs( $fd, $this->xmlTagS( $level, 'contextList' ));
			$htypes = array_keys($vh['context']);
			foreach( $htypes as $htype )
			{
				$ct = &$vh['context'][$htype];
				foreach ( $ct as $ctx )
				{
					fputs( $fd, $this->xmlTagS($level, 'context' ));

					foreach ($ctx as $key=>$val)
					{
						if ( $val !== '' && $key != 'accessControl_allow' && $key != 'accessControl_deny' )
							fputs( $fd, $this->xmlTag( $level, $key, $val ));		
					}
					//// access control
					if ( $ctx['accessControl_allow'] != NULL || $ctx['accessControl_deny'] != NULL  )
					{
						fputs( $fd, $this->xmlTagS($level, 'accessControl' ));
						fputs( $fd, $this->xmlTag( $level, 'allow', $ctx['accessControl_allow']));
						fputs( $fd, $this->xmlTag( $level, 'deny', $ctx['accessControl_deny']));
						fputs( $fd, $this->xmlTagE($level, 'accessControl' ));
					}

					fputs( $fd, $this->xmlTagE( $level, 'context' ));
				}
			}
			fputs( $fd, $this->xmlTagE( $level, 'contextList' ));
		}

		//script handler
		fputs( $fd, $this->xmlTagS( $level, 'scriptHandlerList' ));
		if ( isset($vh['scriptHandler']) && count($vh['scriptHandler']) > 0 )
		{
			$sc = &$vh['scriptHandler'];
			$keys = array_keys($sc);
			sort( $keys );
			foreach( $keys as $key )
			{
				fputs( $fd, $this->xmlTagS( $level, 'scriptHandler' ));
				fputs( $fd, $this->xmlTag(  $level, 'suffix', $sc[$key]['suffix'] ));
				fputs( $fd, $this->xmlTag(  $level, 'type', $sc[$key]['type'] ));
				if ( isset( $sc[$key]['handler'] ))
					fputs( $fd, $this->xmlTag(  $level, 'handler', $sc[$key]['handler'] ));
				fputs( $fd, $this->xmlTagE( $level, 'scriptHandler' ));
			}
		}
		fputs( $fd, $this->xmlTagE( $level, 'scriptHandlerList' ));

		//error pages
		if ( isset($vh['errUrl']) && count($vh['errUrl']) > 0 )
		{
			fputs( $fd, $this->xmlTagS( $level, 'customErrorPages' ));
			$eu = &$vh['errUrl'];
			$keys = array_keys($eu);
			sort( $keys );
			foreach( $keys as $key )
			{
				fputs( $fd, $this->xmlTagS( $level, 'errorPage' ));
				fputs( $fd, $this->xmlTag(  $level, 'errCode', $key ));
				fputs( $fd, $this->xmlTag(  $level, 'url', $eu[$key]['url'] ));
				fputs( $fd, $this->xmlTagE( $level, 'errorPage' ));
			}
			fputs( $fd, $this->xmlTagE( $level, 'customErrorPages' ));
		}

		fputs( $fd, $this->xmlTagE( $level, 'virtualHostConfig' ));
		fclose($fd);
		$this->_isChanged = true;
		clearstatcache();
		$this->_modTime['vh'][$vhname]['file'] = $confFile;
		$this->_modTime['vh'][$vhname]['time'] = filemtime($confFile);

		return true;
	}

	function saveServFile($updateMenu)
	{
		if ( $this->_isAdm )
		{
			$filename = $_SESSION['admRoot'] . "/conf/admin_config.xml" ;
			$this->saveAdminFile($filename, $updateMenu);
		}		
		else
		{
			$filename = $_SERVER['HTTPD_SERVER_ROOT'] . "conf/httpd_config.xml";
			$this->saveServerFile($filename, $updateMenu);
		}
	}

	function saveServerFile($filename, $updateMenu)
	{
		$fd = fopen($filename, 'w');
		if ( !$fd ) 
			return false;
		fputs( $fd, '<?xml version="1.0" encoding="UTF-8"?>'."\n");
		$level = 0;
		fputs( $fd, $this->xmlTagS( $level, 'httpServerConfig' ));

		$g = &$this->_serv['general'];
		fputs( $fd, $this->xmlTag( $level, 'serverName', $g['serverName']));
		fputs( $fd, $this->xmlTag( $level, 'user', $g['user']));
		fputs( $fd, $this->xmlTag( $level, 'group', $g['group']));
		fputs( $fd, $this->xmlTag( $level, 'priority', $g['priority']));
		fputs( $fd, $this->xmlTag( $level, 'chrootPath', $g['chrootPath']));
		fputs( $fd, $this->xmlTag( $level, 'enableChroot', $g['enableChroot']));
		fputs( $fd, $this->xmlTag( $level, 'inMemBufSize', $g['inMemBufSize']));
		fputs( $fd, $this->xmlTag( $level, 'swappingDir', $g['swappingDir']));
		fputs( $fd, $this->xmlTag( $level, 'autoRestart', $g['autoRestart']));
		fputs( $fd, $this->xmlTag( $level, 'adminRoot', $g['adminRoot']));

		fputs( $fd, $this->xmlTag( $level, 'mime', $g['mime']));
		if ( isset($g['indexFiles']) )
			fputs( $fd, $this->xmlTag( $level, 'indexFiles', $g['indexFiles'] ));
		fputs( $fd, $this->xmlTag( $level, 'showVersionNumber', $g['showVersionNumber']));
		fputs( $fd, $this->xmlTag( $level, 'adminEmails', $g['adminEmails']));

		// logging
		fputs( $fd, $this->xmlTagS( $level, 'logging' ));
		fputs( $fd, $this->xmlTagS( $level, 'log' ));
		fputs( $fd, $this->xmlTag(  $level, 'fileName', $g['log_fileName'] ));
		fputs( $fd, $this->xmlTag(  $level, 'logLevel', $g['log_logLevel'] ));
		fputs( $fd, $this->xmlTag(  $level, 'debugLevel', $g['log_debugLevel'] ));
		fputs( $fd, $this->xmlTag(  $level, 'rollingSize', $g['log_rollingSize']));
		fputs( $fd, $this->xmlTag(  $level, 'enableStderrLog', $g['log_enableStderrLog']));
		fputs( $fd, $this->xmlTagE( $level, 'log' ));
		fputs( $fd, $this->xmlTagS( $level, 'accessLog' ));
		fputs( $fd, $this->xmlTag(  $level, 'fileName', $g['accessLog_fileName']));
		fputs( $fd, $this->xmlTag(  $level, 'logReferer', $g['accessLog_logReferer']));
		fputs( $fd, $this->xmlTag(  $level, 'logUserAgent', $g['accessLog_logUserAgent']));
		fputs( $fd, $this->xmlTag(  $level, 'rollingSize', $g['accessLog_rollingSize']));
		fputs( $fd, $this->xmlTag(  $level, 'keepDays', $g['accessLog_keepDays'] ));
		fputs( $fd, $this->xmlTagE( $level, 'accessLog' ));
		fputs( $fd, $this->xmlTagE( $level, 'logging' ));


		//htAccess
		if ( isset($g['allowOverride']) || isset($g['accessFileName']) )
		{ 
			fputs( $fd, $this->xmlTagS( $level, 'htAccess' ));
			fputs( $fd, $this->xmlTag( $level, 'allowOverride', $g['allowOverride'] ));
			fputs( $fd, $this->xmlTag( $level, 'accessFileName', $g['accessFileName'] ));
			fputs( $fd, $this->xmlTagE( $level, 'htAccess' ));
		}

		//expires
		fputs( $fd, $this->xmlTagS( $level, 'expires' ));
		fputs( $fd, $this->xmlTag(  $level, 'enableExpires', $g['enableExpires'] ));
		fputs( $fd, $this->xmlTag(  $level, 'expiresDefault', $g['expiresDefault'] ));
		fputs( $fd, $this->xmlTag(  $level, 'expiresByType', $g['expiresByType'] ));
		fputs( $fd, $this->xmlTagE( $level, 'expires' ));

		//tuning
		fputs( $fd, $this->xmlTagS( $level, 'tuning' ));
		foreach( $this->_serv['tuning'] as $key=>$val )
		{
			fputs( $fd, $this->xmlTag(  $level, $key, $val ));
		}
		fputs( $fd, $this->xmlTagE( $level, 'tuning' ));

		//security
		$s = &$this->_serv['security'];
		fputs( $fd, $this->xmlTagS( $level, 'security' ));

		fputs( $fd, $this->xmlTagS( $level, 'fileAccessControl' ));
		fputs( $fd, $this->xmlTag(  $level, 'followSymbolLink', $s['fileAccessControl/followSymbolLink'] ));
		fputs( $fd, $this->xmlTag(  $level, 'checkSymbolLink', $s['fileAccessControl/checkSymbolLink'] ));
		fputs( $fd, $this->xmlTag(  $level, 'requiredPermissionMask', $s['fileAccessControl/requiredPermissionMask'] ));
		fputs( $fd, $this->xmlTag(  $level, 'restrictedPermissionMask', $s['fileAccessControl/restrictedPermissionMask'] ));
		fputs( $fd, $this->xmlTagE( $level, 'fileAccessControl' ));

		fputs( $fd, $this->xmlTagS( $level, 'CGIRLimit' ));
		fputs( $fd, $this->xmlTag(  $level, 'maxCGIInstances', $s['CGIRLimit/maxCGIInstances'] ));
		fputs( $fd, $this->xmlTag(  $level, 'minUID', $s['CGIRLimit/minUID'] ));
		fputs( $fd, $this->xmlTag(  $level, 'minGID', $s['CGIRLimit/minGID'] ));
		fputs( $fd, $this->xmlTag(  $level, 'priority', $s['CGIRLimit/priority'] ));
		fputs( $fd, $this->xmlTag(  $level, 'CPUSoftLimit', $s['CGIRLimit/CPUSoftLimit'] ));
		fputs( $fd, $this->xmlTag(  $level, 'CPUHardLimit', $s['CGIRLimit/CPUHardLimit'] ));
		fputs( $fd, $this->xmlTag(  $level, 'memSoftLimit', $s['CGIRLimit/memSoftLimit'] ));
		fputs( $fd, $this->xmlTag(  $level, 'memHardLimit', $s['CGIRLimit/memHardLimit'] ));
		fputs( $fd, $this->xmlTag(  $level, 'procSoftLimit', $s['CGIRLimit/procSoftLimit'] ));
		fputs( $fd, $this->xmlTag(  $level, 'procHardLimit', $s['CGIRLimit/procHardLimit'] ));
		fputs( $fd, $this->xmlTagE( $level, 'CGIRLimit' ));

		fputs( $fd, $this->xmlTagS( $level, 'perClientConnLimit' ));
		fputs( $fd, $this->xmlTag(  $level, 'throttleLimit', $s['perClientConnLimit/throttleLimit'] ));
		fputs( $fd, $this->xmlTag(  $level, 'softLimit', $s['perClientConnLimit/softLimit'] ));
		fputs( $fd, $this->xmlTag(  $level, 'hardLimit', $s['perClientConnLimit/hardLimit'] ));
		fputs( $fd, $this->xmlTag(  $level, 'gracePeriod', $s['perClientConnLimit/gracePeriod'] ));
		fputs( $fd, $this->xmlTag(  $level, 'banPeriod', $s['perClientConnLimit/banPeriod'] ));
		fputs( $fd, $this->xmlTagE( $level, 'perClientConnLimit' ));

		fputs( $fd, $this->xmlTagS( $level, 'accessDenyDir' ));
		if ( isset($s['accessDenyDir']) )
		{
			foreach( $s['accessDenyDir'] as $value )
			{
				fputs( $fd, $this->xmlTag( $level, 'dir', $value ));
			}
		}
		fputs( $fd, $this->xmlTagE($level, 'accessDenyDir' ));
		fputs( $fd, $this->xmlTagS($level, 'accessControl' ));
		if ( isset($s['accessControl_allow']) )
			fputs( $fd, $this->xmlTag( $level, 'allow', $s['accessControl_allow']));
		if ( isset($s['accessControl_deny']))
			fputs( $fd, $this->xmlTag( $level, 'deny', $s['accessControl_deny']));
		fputs( $fd, $this->xmlTagE($level, 'accessControl' ));
		fputs( $fd, $this->xmlTagE($level, 'security' ));

		//ext
		fputs( $fd, $this->xmlTagS( $level, 'extProcessorList' ));
		if ( isset($this->_serv['ext']) && count($this->_serv['ext']) > 0)
		{
			$ext0 = &$this->_serv['ext'];
			$ext = array();
			foreach( $ext0 as $a )
			{
				$id = $a['type'] . $a['name'];
				$ext[$id] = $a;
			}
			ksort($ext);
			reset($ext);

			foreach( $ext as $f )
			{
				fputs( $fd, $this->xmlTagS( $level, 'extProcessor' ));
				fputs( $fd, $this->xmlTag(  $level, 'name', $f['name'] ));
				fputs( $fd, $this->xmlTag(  $level, 'address', $f['address'] ));
				fputs( $fd, $this->xmlTag(  $level, 'type', $f['type'] ));
				fputs( $fd, $this->xmlTag(  $level, 'maxConns', $f['maxConns'] ));
				fputs( $fd, $this->xmlTag(  $level, 'initTimeout', $f['initTimeout'] ));
				fputs( $fd, $this->xmlTag(  $level, 'retryTimeout', $f['retryTimeout'] ));
				if ( isset( $f['env'] ) )
				{
					foreach( $f['env'] as $value )
					{
						fputs( $fd, $this->xmlTag(  $level, 'env', $value ));
					}
				}
				if ( $f['type'] == 'fcgi' )
				{
					fputs( $fd, $this->xmlTag(  $level, 'autoStart', $f['autoStart'] ));
					fputs( $fd, $this->xmlTag(  $level, 'path', $f['path'] ));
					fputs( $fd, $this->xmlTag(  $level, 'backlog', $f['backlog'] ));
					fputs( $fd, $this->xmlTag(  $level, 'instances', $f['instances'] ));
				}
				fputs( $fd, $this->xmlTagE( $level, 'extProcessor' ));
			}
		}
		fputs( $fd, $this->xmlTagE( $level, 'extProcessorList' ));

		//script handler
		fputs( $fd, $this->xmlTagS( $level, 'scriptHandlerList' ));
		if ( isset($this->_serv['scriptHandler']) && count($this->_serv['scriptHandler']) > 0)
		{
			$sc = &$this->_serv['scriptHandler'];
			$keys = array_keys($sc);
			sort( $keys );
			foreach( $keys as $key )
			{
				fputs( $fd, $this->xmlTagS( $level, 'scriptHandler' ));
				fputs( $fd, $this->xmlTag(  $level, 'suffix', $sc[$key]['suffix'] ));
				fputs( $fd, $this->xmlTag(  $level, 'type', $sc[$key]['type'] ));
				if ( isset( $sc[$key]['handler'] ))
					fputs( $fd, $this->xmlTag(  $level, 'handler', $sc[$key]['handler'] ));
				fputs( $fd, $this->xmlTagE( $level, 'scriptHandler' ));
			}
		}
		fputs( $fd, $this->xmlTagE( $level, 'scriptHandlerList' ));

		//listner
		fputs( $fd, $this->xmlTagS( $level, 'listenerList' ));
		$lnames = array_keys($this->_serv['listeners']);
		sort($lnames);
		reset($lnames);
		foreach( $lnames as $ln )
		{
			$l = &$this->_serv['listeners'][$ln];
			fputs( $fd, $this->xmlTagS( $level, 'listener' ));
			fputs( $fd, $this->xmlTag(  $level, 'name', $ln ));
			$ip = $l['ip'];
			if ( $ip == 'ANY' )
				$ip = '*';
			$addr = $ip . ':' . $l['port'];
			fputs( $fd, $this->xmlTag(  $level, 'address', $addr ));
			fputs( $fd, $this->xmlTag(  $level, 'secure', $l['secure'] ));
			fputs( $fd, $this->xmlTag( $level, 'certFile', $l['certFile'] ));
			fputs( $fd, $this->xmlTag( $level, 'keyFile', $l['keyFile'] ));
			fputs( $fd, $this->xmlTag( $level, 'ciphers', $l['ciphers'] ));
			fputs( $fd, $this->xmlTagS( $level, 'vhostMapList' ));
			if ( isset($l['vhmap']) && count($l['vhmap']) > 0)
			{
				$keys = array_keys( $l['vhmap'] );
				sort( $keys );
				foreach ( $keys as $key )
				{
					fputs( $fd, $this->xmlTagS($level, 'vhostMap' ));
					fputs( $fd, $this->xmlTag( $level, 'vhost', $key ));
					fputs( $fd, $this->xmlTag( $level, 'domain', $l['vhmap'][$key]['domain'] ));
					fputs( $fd, $this->xmlTagE($level, 'vhostMap' ));
				}
			}
			fputs( $fd, $this->xmlTagE( $level, 'vhostMapList' ));
			fputs( $fd, $this->xmlTagE( $level, 'listener' ));
		}
		fputs( $fd, $this->xmlTagE( $level, 'listenerList' ));

		//vh
		fputs( $fd, $this->xmlTagS( $level, 'virtualHostList' ));
		$hnames = array_keys($this->_serv['vhTop']);
		sort( $hnames);
		foreach( $hnames as $hn )
		{
			if ( $hn )
			{
				$vh = &$this->_serv['vhTop'][$hn];
				fputs( $fd, $this->xmlTagS( $level, 'virtualHost' ));
				fputs( $fd, $this->xmlTag(  $level, 'name', $hn ));
				if ( substr($vh['vhRoot'],-1) != '/' )
					$vh['vhRoot'] .= '/';
				fputs( $fd, $this->xmlTag(  $level, 'vhRoot', $vh['vhRoot'] ));
				fputs( $fd, $this->xmlTag(  $level, 'configFile', $vh['configFile'] ));
				fputs( $fd, $this->xmlTag(  $level, 'allowSymbolLink', $vh['allowSymbolLink'] ));
				fputs( $fd, $this->xmlTag(  $level, 'enableScript', $vh['enableScript'] ));
				fputs( $fd, $this->xmlTag(  $level, 'restrained', $vh['restrained'] ));
				fputs( $fd, $this->xmlTag(  $level, 'setUIDMode', $vh['setUIDMode'] ));
				fputs( $fd, $this->xmlTag(  $level, 'chrootMode', $vh['chrootMode'] ));
				fputs( $fd, $this->xmlTag(  $level, 'chrootPath', $vh['chrootPath'] ));
				fputs( $fd, $this->xmlTagE( $level, 'virtualHost' ));
			}
		}
		fputs( $fd, $this->xmlTagE( $level, 'virtualHostList' ));

		fputs( $fd, $this->xmlTagE( $level, 'httpServerConfig' ));
		fclose($fd);

		if ( $updateMenu )
			$this->genMenuInfo();
		$this->_isChanged = true;
		clearstatcache();
		$this->_modTime['serv']['file'] = $filename;
		$this->_modTime['serv']['time'] = filemtime($filename);
		return true;
	}

	function saveAdminFile($filename, $updateMenu)
	{
		$fd = fopen($filename, 'w');
		if ( !$fd ) 
			return false;
		fputs( $fd, '<?xml version="1.0" encoding="UTF-8"?>'."\n");
		$level = 0;
		fputs( $fd, $this->xmlTagS( $level, 'adminConfig' ));

		$g = &$this->_serv;
		fputs( $fd, $this->xmlTag( $level, 'enableCoreDump', $g['enableCoreDump' ] ));
		// logging
		fputs( $fd, $this->xmlTagS( $level, 'logging' ));
		fputs( $fd, $this->xmlTagS( $level, 'log' ));
		fputs( $fd, $this->xmlTag(  $level, 'useServer', $g['log_useServer'] ));
		fputs( $fd, $this->xmlTag(  $level, 'fileName', $g['log_fileName'] ));
		fputs( $fd, $this->xmlTag(  $level, 'logLevel', $g['log_logLevel'] ));
		fputs( $fd, $this->xmlTag(  $level, 'rollingSize', $g['log_rollingSize']));
		fputs( $fd, $this->xmlTagE( $level, 'log' ));
		fputs( $fd, $this->xmlTagS( $level, 'accessLog' ));
		fputs( $fd, $this->xmlTag(  $level, 'useServer', $g['accessLog_useServer']));
		fputs( $fd, $this->xmlTag(  $level, 'fileName', $g['accessLog_fileName']));
		fputs( $fd, $this->xmlTag(  $level, 'logReferer', $g['accessLog_logReferer']));
		fputs( $fd, $this->xmlTag(  $level, 'logUserAgent', $g['accessLog_logUserAgent']));
		fputs( $fd, $this->xmlTag(  $level, 'rollingSize', $g['accessLog_rollingSize']));
		fputs( $fd, $this->xmlTag(  $level, 'keepDays', $g['accessLog_keepDays'] ));
		fputs( $fd, $this->xmlTagE( $level, 'accessLog' ));
		fputs( $fd, $this->xmlTagE( $level, 'logging' ));

		//security
		$s = &$this->_serv;
		fputs( $fd, $this->xmlTagS( $level, 'security' ));
		fputs( $fd, $this->xmlTagS($level, 'accessControl' ));
		if ( isset($s['accessControl_allow']) ) 
			fputs( $fd, $this->xmlTag( $level, 'allow', $s['accessControl_allow']));
		if ( isset($s['accessControl_deny']) ) 
			fputs( $fd, $this->xmlTag( $level, 'deny', $s['accessControl_deny']));
		fputs( $fd, $this->xmlTagE($level, 'accessControl' ));
		fputs( $fd, $this->xmlTagE($level, 'security' ));

		//listner
		fputs( $fd, $this->xmlTagS( $level, 'listenerList' ));
		$lnames = array_keys($this->_serv['listeners']);
		sort($lnames);
		reset($lnames);
		foreach( $lnames as $ln )
		{
			$l = &$this->_serv['listeners'][$ln];
			fputs( $fd, $this->xmlTagS( $level, 'listener' ));
			fputs( $fd, $this->xmlTag(  $level, 'name', $ln ));
			$ip = $l['ip'];
			if ( $ip == 'ANY' )
				$ip = '*';
			$addr = $ip . ':' . $l['port'];
			fputs( $fd, $this->xmlTag(  $level, 'address', $addr ));
			fputs( $fd, $this->xmlTag(  $level, 'secure', $l['secure'] ));
			if ( $l['secure'] )
			{
				fputs( $fd, $this->xmlTag( $level, 'certFile', $l['certFile'] ));
				fputs( $fd, $this->xmlTag( $level, 'keyFile', $l['keyFile'] ));
				fputs( $fd, $this->xmlTag( $level, 'ciphers', $l['ciphers'] ));
			}
			fputs( $fd, $this->xmlTagE( $level, 'listener' ));
		}
		fputs( $fd, $this->xmlTagE( $level, 'listenerList' ));

		fputs( $fd, $this->xmlTagE( $level, 'adminConfig' ));
		fclose($fd);

		if ( $updateMenu )
			$this->genMenuInfo();

		$this->_isChanged = true;
		clearstatcache();
		$this->_modTime['adm']['file'] = $filename;
		$this->_modTime['adm']['time'] = filemtime($filename);
		return true;
	}

	function xmlTag($level, $tag, $value)
	{
		return str_repeat('  ', $level) . "<$tag>$value</$tag>\n";
	}
	function xmlTagS(&$level, $tag)
	{
		return str_repeat('  ', $level++) . "<$tag>\n";
	}
	function xmlTagE(&$level, $tag)
	{
		return str_repeat('  ', --$level) . "</$tag>\n";
	}

	function &getServNames($tag, $id)
	{
		$names = array();

		if ( $tag == 'scr' )
		{
			$t = array_keys( $this->_serv['scriptHandler'] );
			foreach( $t as $value )
			{
				if ( $value != $id )
				{
					$vals = DAttr::splitMultiple($value);
					$names = array_merge( $names, $vals );
				}
			}
		}
		else
		{
			$t = array_keys( $this->_serv[$tag] );
			foreach( $t as $value )
			{
				if ( $value != $id )
					$names[] = $value;
			}
		}
		return $names;
	}

	function &getVhNames( $tag, &$vh, $id )
	{
		$names = array();
		if ( $tag == 'scr' )
		{
			if ( isset($vh['scriptHandler']) && count($vh['scriptHandler']) > 0 )
			{
				$t = array_keys( $vh['scriptHandler'] );
				foreach( $t as $value )
				{
					if ( $value != $id )
					{
						$vals = DAttr::splitMultiple($value);
						$names = array_merge( $names, $vals );
					}
				}
			}
			return $names;
		}
		if ( $tag == 'err' )
		{
			if ( isset($vh['errUrl']) && count($vh['errUrl']) > 0 )
				$t = array_keys( $vh['errUrl'] );
		}
		elseif ( $tag == 'hta' )
		{
			if ( isset($vh['security']['realm']) && count($vh['security']['realm']) )
				$t = array_keys( $vh['security']['realm'] );
		}
		elseif ( $tag == 'Vext' )
		{
			if ( isset($this->_serv['ext']) && count($this->_serv['ext']) > 0 )
				$names = array_keys( $this->_serv['ext'] );
			if ( isset($vh['ext']) && count($vh['ext']) > 0 )
				$t = array_keys( $vh['ext'] );
		}
		elseif ( $tag == 'Vctx' )
		{
			$t = array();
			if ( isset($vh['context']['NULL']) && count($vh['context']['NULL']) > 0 )
				$t = array_merge( $t, array_keys($vh['context']['NULL']) );
			if ( isset($vh['context']['webapp']) && count($vh['context']['webapp']) > 0 )
				$t = array_merge( $t, array_keys($vh['context']['webapp']) );
			if ( isset($vh['context']['servlet']) && count($vh['context']['servlet']) > 0 )
				$t = array_merge( $t, array_keys($vh['context']['servlet']) );
			if ( isset($vh['context']['fcgi']) && count($vh['context']['fcgi']) > 0 )
				$t = array_merge( $t, array_keys($vh['context']['fcgi']) );
			if ( isset($vh['context']['proxy']) && count($vh['context']['proxy']) > 0 )
				$t = array_merge( $t, array_keys($vh['context']['proxy']) );
			if ( isset($vh['context']['cgi']) && count($vh['context']['cgi']) > 0 )
				$t = array_merge( $t, array_keys($vh['context']['cgi']) );
			if ( isset($vh['context']['redirect']) && count($vh['context']['redirect']) > 0 )
				$t = array_merge( $t, array_keys($vh['context']['redirect']) );
			if ( count($t) == 0 )
				unset( $t );
		}

		if ( isset($t) )
		{
			foreach( $t as $value )
			{
				if ( $value != $id )
					$names[] = $value;
			}
		}

		return $names;
	}

	function refreshExtApp($vhname, &$from)
	{
		$ext['fcgi'] = array();
		$ext['servlet'] = array();
		$ext['proxy'] = array();
		if ( isset($this->_serv['ext']) )
		{
			foreach( $this->_serv['ext'] as $e )
			{
				if ( $e['type'] == 'fcgi' )
					$ext['fcgi']['s'][] = $e['name'];
				elseif( $e['type'] == 'servlet' )
					$ext['servlet']['s'][] = $e['name'];
				elseif( $e['type'] == 'proxy' )
					$ext['proxy']['s'][] = $e['name'];
			}
		}

		if ( $vhname && isset($this->_vhosts[$vhname]['ext']) )
		{
			$vh = &$this->_vhosts[$vhname];
			foreach( $vh['ext'] as $e )
			{
				if ( $e['type'] == 'fcgi' )
					$ext['fcgi']['v'][] = $e['name'];
				elseif ( $e['type'] == 'servlet' )
					$ext['servlet']['v'][] = $e['name'];
				elseif ( $e['type'] == 'proxy' )
					$ext['proxy']['v'][] = $e['name'];
			}
		}
		$shandlers = array();
		if ( isset( $ext['servlet']['s'] ))
		{
			foreach($ext['servlet']['s'] as $se)
				$shandlers[$se] = "[Server Level]: $se"; 
		}
		if ( isset( $ext['servlet']['v'] ))
		{
			foreach($ext['servlet']['v'] as $se)
				$shandlers[$se] = "[VHost Level]: $se"; 
		}
		$fhandlers = array();
		if ( isset( $ext['fcgi']['s'] ))
		{
			foreach($ext['fcgi']['s'] as $se)
				$fhandlers[$se] = "[Server Level]: $se"; 
		}
		if ( isset( $ext['fcgi']['v'] ))
		{
			foreach($ext['fcgi']['v'] as $se)
				$fhandlers[$se] = "[VHost Level]: $se"; 
		}
		$phandlers = array();
		if ( isset( $ext['proxy']['s'] ))
		{
			foreach($ext['proxy']['s'] as $se)
				$phandlers[$se] = "[Server Level]: $se"; 
		}
		if ( isset( $ext['proxy']['v'] ))
		{
			foreach($ext['proxy']['v'] as $se)
				$phandlers[$se] = "[VHost Level]: $se"; 
		}
		$from['servletHandlers'] = $shandlers;
		$from['fcgiHandlers'] = $fhandlers;
		$from['webServers'] = $phandlers;
	}

	function refreshVhRealms($vhname, &$from)
	{
		$realms = array();
		$realms[''] = '';
		if ( isset( $this->_vhosts[$vhname]['security']['realm']) )
		{
			$rn = array_keys($this->_vhosts[$vhname]['security']['realm']);
			foreach ( $rn as $name )
				$realms[$name] = $name;
		}
		$from['realms'] = $realms;
	}

	function refreshVhRoot($vhname)
	{
		$vhroot = $this->getAbsFile('$VH_ROOT/', 'VR', $vhname);
		$this->_vhosts[$vhname]['VH_ROOT'] = $vhroot;
		$vhconf = $this->getAbsFile($this->_serv['vhTop'][$vhname]['configFile'], 'VR', $vhname);
		if ( !file_exists($vhconf) )
			$this->saveVhFile($vhname);

		clearstatcache();
		$this->_modTime['vh'][$vhname]['time'] = filemtime($vhconf);
		$this->_modTime['vh'][$vhname]['file'] = $root;
	}

	function refreshVhDocRoot($vhname)
	{
		$docroot = $this->getAbsFile('$DOC_ROOT/', 'DR', $vhname);
		$this->_vhosts[$vhname]['DOC_ROOT'] = $docroot;
	}
}
	
?>