<?php

class Service
{
	var $cmd;
	var $cmdStatus;
	var $act;
	var $actId;
	var $FNAME = '/tmp/lshttpd/.admin';
	var $FSTATUS = '/tmp/lshttpd/.status';
	var $FPID = '/tmp/lshttpd/lshttpd.pid';
	var $serverLastModTime = 100;
	var $listeners = array();
	var $adminL = array();
	var $vhosts = array();
	var $serv = array();

	function init()
	{
		clearstatcache();
		$this->serverLastModTime = filemtime($this->FSTATUS);
		$this->readStatus();
	}

	function refreshConf(&$data)
	{
		$listeners = $data['listener'];
		foreach( $listeners as $lname => $addr )
		{
			if ( !isset($this->listeners[$lname]) )
			{
				$this->listeners[$lname] = array();
				$this->listeners[$lname]['addr'] = $addr;
				$this->listeners[$lname]['status'] = 'Error';
			}
		}

		$vhnames = $data['vhost'];
		foreach( $this->vhosts as $key => $value )
		{
			$this->vhosts[$key] = $value{0} . 'D';
		}
		foreach( $vhnames as $vhname )
		{
			if ( !isset($this->vhosts[$vhname]) )
				$this->vhosts[$vhname] = '0A';
			else
				$this->vhosts[$vhname] = $this->vhosts[$vhname]{0} . 'N';
		}

		$this->serv['name'] = $data['serv'];
		$fd = fopen($this->FPID, 'r');
		if ( $fd )
		{
			$this->serv['pid'] = trim(fgets($fd));
			fclose($fd);
		}
	}

	function process($act, $actId)
	{
		$this->act = $act;
		$this->actId = $actId;

		if ( $this->isPending() )
			return false;

		$this->checkLastMod();

		if ( $actId )
			$this->vhostControl();
		elseif ( $act == 'restart' )
			$this->restartServer();
		elseif ( $act == 'reload' )
			$this->reloadServer();
	}

	function waitForChange()
	{
		for( $count = 0; $count < 5; ++$count)
		{
			if ( !$this->checkLastMod() )
				sleep(1);
			else
				return true;
		}
		return false;

	}

	function readStatus()
	{
		$this->listeners = array();
		$this->adminL = array();
		$this->vhosts = array();
		$fd = fopen($this->FSTATUS, 'r');
		if ( !$fd )
			return false; 
		while ( !feof($fd) )
		{
			$buffer = fgets($fd, 512);
			if ( strncmp($buffer, 'LISTENER0', 9) == 0 )
			{
				$this->readListener($this->listeners, $buffer, $fd);
			}
			elseif ( strncmp($buffer, 'LISTENER1', 9) == 0 )
			{
				$this->readListener($this->adminL, $buffer, $fd);
			}
			elseif ( strncmp($buffer, 'VHOST', 5) == 0 )
			{
				$this->readVh($buffer, $fd);
			}
			elseif ( strncmp($buffer, 'EOF', 3) == 0 )
				break;
		}
		fclose($fd);

	}

	function readListener(&$l, $buffer, &$fd)
	{
		if ( preg_match( "/\[(.+)\] (.+)$/", $buffer, $matches ))
		{
			$lname = $matches[1];
			$l[$lname]['addr'] = $matches[2];
			$l[$lname]['status'] = 'Running';
			$tmp = fgets($fd, 512);
			while ( strncmp($tmp, 'ENDL' ,4) != 0 )
			{
				if ( strncmp( $tmp, 'LVMAP', 5 ) == 0)
				{
					if ( preg_match( "/\[(.+)\] (.+)$/", $tmp, $tm ))
					{
						$l[$lname]['map'][$tm[1]][] = $tm[2];
					}
				}
				$tmp = fgets($fd, 512);
			}
		}
	}

	function readVh($buffer, &$fd)
	{
		if ( preg_match( "/\[(.+)\] ([01])/", $buffer, $m ))
		{
			$vname = $m[1];
			if ( $vname != '_AdminVHost' )
				$this->vhosts[$m[1]] = $m[2];
		}
	}

	function checkLastMod()
	{
		clearstatcache();
		$mt = filemtime($this->FSTATUS);
		if ( $this->serverLastModTime != $mt )
		{
			$this->serverLastModTime = $mt;
			return true;
		}
		else
		{
			return false;
		}
	}
	
	function restartServer()
	{
		unset($this->cmd);
		$this->cmd[] = 'restart';
		$this->issueCmd();
	}

	function reloadServer()
	{
		unset($this->cmd);
		$this->cmd[] = 'reload:config';

		$this->issueCmd();
		$this->waitForChange();
		$this->readStatus();
	}

	function vhostControl()
	{
		unset( $this->cmd );
		$this->cmd[] = "$this->act:vhost:$this->actId";
		$this->issueCmd();
		$this->waitForChange();
		$this->readStatus();
	}

	function issueCmd()
	{
		$fd = fopen($this->FNAME, 'w');
		if ( !$fd )
		{
			$this->cmdStatus = 'FAIL';
			return false;
		}
		foreach( $this->cmd as $line )
		{
			fputs( $fd, $line );
		}
		fputs( $fd, 'end of actions' );
		fclose($fd);
	}

	function isPending()
	{
		if ( file_exists($this->FNAME) )
		{
			$this->cmdStatus = 'IS_PENDING';
			return true;
		}
		else
			return false;
	}

	function &getLogData($filename)
	{
		$data = array();
		$data['filename'] = $filename;
		$level = $_GET['sel_level'];
		if ( $level == NULL )
			$level = 'I';
		$data['level'] = $level;

		$fd = fopen($filename, 'r');
		if ( !$fd )
			return $data;

		fseek( $fd, 0, SEEK_END );
		$endpos = ftell($fd);
		$data['logSize'] = number_format( $endpos/1024, 2);

		$searchSize = (int) $_GET['searchSize'];
		if ( $searchSize == 0 )
			$searchSize = 20;

		$data['searchSize'] = $searchSize;

		$searchFrom = (int)$_GET['searchFrom'];

		if ( isset($_GET['end_x']) )
		{
			$searchFrom = $endpos;
		}
		else
		{
			if ( isset($_GET['prev_x']) )
				$searchFrom -= $searchSize;
			elseif ( isset($_GET['next_x']) )
				$searchFrom += $searchSize;
			elseif ( isset($_GET['begin_x']) )
				$searchFrom = 0;

			if ( $searchFrom < 0 )
				$searchFrom = 0;

			$data['searchFrom'] = $searchFrom;
			$searchFrom *= 1024;
		}

		$searchSize *= 1024;

		if ( $searchFrom >= $endpos )
		{
			$searchFrom = $endpos - $searchSize;
			if ( $searchFrom < 0 )
				$searchFrom = 0;
			$data['searchFrom'] = number_format( $searchFrom/1024, 2);
		}
		if ( $searchFrom + $searchSize < $endpos )
			$endpos = $searchFrom + $searchSize;
		fclose($fd);

		$data['fromPos'] = $searchFrom;
		$data['endPos'] = $endpos;
		return $data;
	}


	function showLog(&$data)
	{
		$newlineTag = '[ERR[WAR[NOT[INF[DEB';
		$levels = array( 'E' => 1, 'W' => 2, 'N' => 3, 'I' => 4, 'D' => 5 );
		$level = $levels[$data['level']{0}];

		$fd = fopen($data['filename'], 'r');
		if ( !$fd )
		{
			echo '<tr class="error"><td colspan=3>Failed read server log file from '.$data['filename'].'</td></tr>';
		}
		fseek( $fd, 0, SEEK_END );
		$endpos = $data['endPos'];
		fseek( $fd, $data['fromPos'] );
		$start = 0;
		$result = '';
		$totalLine = 0;
		$line = 0;
		$cutline = 0;
		$buffer = fgets($fd);
		while ( !preg_match("/^\d{4}-\d{2}-\d{2} /", $buffer) )
		{	$buffer = fgets($fd); }

		do
		{
			$buffer = chop($buffer);
			// check if new line
			$c25 = substr($buffer, 25 , 3);
			if ( $c25 && strstr($newlineTag, $c25) )
			{
				// is new line 
				$totalLine ++;
				if ( $start )
				{
					// finish prior line
					$result .= '</td></tr>' . "\n";
					$start = 0;
				}
				$b25 = $c25{0};
				if ( $levels[$b25] <= $level )
				{
					// start a new line
					$start = 1;
					$line ++;
					$style = 'log_' . $b25;
					$result .= '<tr><td class="'. $style . '0">'. substr($buffer, 0, 23);
					$result .= '</td><td class="'. $style . '1">';
					$i = strpos($buffer, ' ', 24);
					$result .= ( substr($buffer, 25, $i - 26) );
					$result .= '</td><td class="'. $style . '2">';
					$result .= htmlspecialchars( substr($buffer, $i+1) );
					if ( ++$cutline > 350 )
					{
						$cutline = 0;
						echo $result;
						$result = '';
					}
				}
			}
			elseif ( $start )
			{
				// multi-line output
				$result .= '<br>'.$buffer;
			}

			$curpos = ftell($fd);
			if ( $curpos >= $endpos )
				break;

		} while ( $buffer = fgets($fd) );

		fclose($fd);
		if ( $start )
			$result .= '</td></tr>'."\n";
		$result .= '<tr class="log-trailer"><td colspan="3">Retrieved '. number_format($line) . ' messages of total ' . number_format($totalLine) . ' messages searched.</td></tr>';

		echo $result;

	}

	/*function dbg_out($tag, &$obj)
	{
		echo "<!-- $tag --\n";
		var_dump($obj);
		echo "-->\n";
	}
	function dbg_tag($tag)
	{
		echo "<!-- $tag -->\n";
	}*/


}

?>
