<?php
class XmlTreeBuilder
{ 
	var $parser; 
	var $node_stack;

	function XmlTreeBuilder($xmlstring='') 
	{ 
		if ($xmlstring) 
			return($this->parse($xmlstring)); 
		return(true); 
	} 

	function &parse($xmlstring='') 
	{ 
		$this->parser = xml_parser_create(); 
		xml_set_object($this->parser, $this); 
		xml_parser_set_option($this->parser, XML_OPTION_CASE_FOLDING, false); 
		xml_set_element_handler($this->parser, 'startElement', 'endElement'); 
		xml_set_character_data_handler($this->parser, 'characterData'); 
		
		// Build a Root node and initialize the node_stack... 
		$this->node_stack = array(); 
		$this->startElement(null, 'root', array()); 
		
		// parse the data and free the parser... 
		xml_parse($this->parser, $xmlstring); 
		xml_parser_free($this->parser); 
		
		// recover the root node from the node stack 
		$rnode = array_pop($this->node_stack); 
		
		// return the root node... 
		return($rnode['ELEMENTS'][0]); 
	} 

	function startElement($parser, $name, $attrs) 
	{ 
		$node = array(); 
		$node['TAG'] = $name;
		
		foreach ($attrs as $key => $value) 
		{ 
			$node[$key] = $value; 
		} 
		
		$node['VALUE'] = ''; 
		$node['ELEMENTS'] = array(); 
		
		// add the new node to the end of the node stack 
		array_push($this->node_stack, $node); 
	} 
	
	function endElement($parser, $name) 
	{ 
		// pop this element off the node stack 
		$node = array_pop($this->node_stack); 
		$node['VALUE'] = trim($node['VALUE']); 
		
		// and add it an element of the last node in the stack... 
		$lastnode = count($this->node_stack); 
		array_push($this->node_stack[$lastnode-1]['ELEMENTS'], $node); 
	} 
	
	function characterData($parser, $data) 
	{ 
		// add this data to the last node in the stack... 
		$lastnode = count($this->node_stack); 
		$this->node_stack[$lastnode-1]['VALUE'] .= $data; 
	}
 
	function xmlNodeOutput(&$node, $space)
	{
		$output = str_repeat('  ', $space);
		$attrs = array();
		foreach( array_keys($node) as $key )
		{
			if ( $key == 'TAG' )
			{
				$name = &$node['TAG'];
			}
			elseif ( $key == 'VALUE' )
			{
				$data = &$node['VALUE'];
			}
			elseif ( $key == 'ELEMENTS' )
			{
				$elements = &$node['ELEMENTS'];
			}
			else
			{
				$attrs[$key] = $node[$key];
			}

		}
		$name = $node['TAG'];
		$output .= "<$name";
		if ( count($attrs) > 0 )
		{
			foreach( $attrs as $key => $value )
			{
				$output .= " $key=\"$value\"";
			}
		}
		$output .= '>';
		
		if ( count($elements) == 0 )
		{
			$output .= "$data</$name>\n";
		}
		else
		{
			$output .= "\n";
			if ( $data )
			{
				$output .= str_repeat('  ', $space+1);
				$output .= $data . "\n";
			}

			foreach( array_keys($elements) as $key )
			{
				$output .= $this->xmlNodeOutput($elements[$key], $space+1);
			}

			$output .= str_repeat('  ', $space);
			$output .= "</$name>\n"; 
			
		}
		return $output ;
		
	}


} 
?>