#!/bin/sh

LSINSTALL_DIR=`dirname "$0"`
cd $LSINSTALL_DIR
LSINSTALL_DIR=`pwd`

export LSINSTALL_DIR

INST_UTIL=$LSINSTALL_DIR/inst-util.sh

DIR_MOD=755
SDIR_MOD=700
EXEC_MOD=500
CONF_MOD=600
DOC_MOD=644

INST_USER=`id`
INST_USER=`expr "$INST_USER" : 'uid=.*(\(.*\)) gid=.*'`

if [ "x`uname -s`" = "xFreeBSD" ]; then
	PS_CMD="ps -ax"
	ID_GROUPS="id"
else
	PS_CMD="ps -ef"
	ID_GROUPS="id -a"
fi


more ./LICENSE
cat <<EOF

IMPORTANT: In order to continue installation you must agree with above 
           license terms by typing "Yes" with capital "Y"! 

EOF
SUCC=0
TRY=1
while [ $SUCC -eq "0" ]; do
	printf "%s" "Do you agree with above license? "  
	read YES_NO
	if [ "x$YES_NO" != "xYes" ]; then
		if [ $TRY -lt 3 ]; then
			echo "Sorry, wrong answer! Type 'Yes' with capital 'Y', try again!"
			TRY=`expr $TRY + 1`
		else
			echo "Abort installation!" 
			exit 0
		fi

	else
		SUCC=1
	fi
done

# Get destination directory

SUCC=0
INSTALL_TYPE="reinstall"
SET_LOGIN=1

if [ $INST_USER = "root" ]; then
	DEST_RECOM="/opt/lsws"
	WS_USER="nobody"
else 
	cat <<EOF

As you are not the 'root' user, you may not be able to install the
web server into a system directory or enable chroot, the web server 
process will run on behalf of current user - '$INST_USER'.

EOF
	WS_USER=$INST_USER
	DEST_RECOM="~/lsws"
fi



while [ $SUCC -eq "0" ];  do
	cat <<EOF

Please specify the destination directory. You must have permissions to 
create and manage the directory. It is recommended to install the web server 
at /opt/lsws, /usr/local/lsws or in your home directory like '~/lsws'.

ATTENTION: The user '$WS_USER' must be able to access the destination
           directory.

EOF
	printf "%s" "Destination [$DEST_RECOM]: "
	read TMP_DEST
	echo ""
	if [ "x$TMP_DEST" = "x" ]; then
		TMP_DEST=$DEST_RECOM
	fi
	if [ `expr "$TMP_DEST" : '~'` -gt 0 ]; then 
		LSWS_HOME="$HOME`echo $TMP_DEST | sed 's/^~//' `"
	else
		LSWS_HOME=$TMP_DEST
	fi
	if [ `expr "$LSWS_HOME" : '\/'` -eq 0 ]; then
		echo "[ERROR] Must be absolute path!"
	else
		SUCC=1
	fi
	if [ ! -d "$LSWS_HOME" ]; then
		mkdir "$LSWS_HOME"
		if [ ! $? -eq 0 ]; then
			SUCC=0
		fi
	else
		cat <<EOF

The destination directory $LSWS_HOME exists. 
To upgrade, press 'Enter', current configuration will not be changed.
To reinstall, press 'R' or 'r'.
To change directory, press 'C' or 'c'. 

EOF

		printf "%s" "Would you like to Upgrade, Reinstall or Change directory [U/r/c]? "
		read TMP_URC
		echo ""
		if [ "x$TMP_URC" = "x" ]; then
			INSTALL_TYPE="upgrade"
			SET_LOGIN=0
		else 
			if [ `expr "$TMP_URC" : '[Uu]'` -gt 0 ]; then
				INSTALL_TYPE="upgrade"
				SET_LOGIN=0
			else 
				if [ `expr "$TMP_URC" : '[Rr]'` -gt 0 ]; then
					INSTALL_TYPE="reinstall"
					SET_LOGIN=1
				else 
					#if [ `expr "$TMP_URC" : '[Cc]'` -gt 0 ]; then
					SUCC=0
				fi
			fi
		fi

	fi

done

export LSWS_HOME

if [ -f "$LSWS_HOME/conf/httpd_config.xml" ]; then
	OLD_USER_CONF=`grep "<user>" "$LSWS_HOME/conf/httpd_config.xml"`
	OLD_GROUP_CONF=`grep "<group>" "$LSWS_HOME/conf/httpd_config.xml"`
	OLD_USER=`expr "$OLD_USER_CONF" : '.*<user>\(.*\)</user>.*'`
	OLD_GROUP=`expr "$OLD_GROUP_CONF" : '.*<group>\(.*\)</group>.*'`
	if [ "x$OLD_USER" != "x" ]; then
	    WS_USER=$OLD_USER
	fi
	if [ "x$OLD_GROUP" != "x" ]; then
		WS_GROUP=$OLD_GROUP
	else
		D_GROUP=`$ID_GROUPS $WS_USER`
		WS_GROUP=`expr "$D_GROUP" : '.*gid=[0-9]*(\(.*\)) groups=.*'`
	fi
else
	INSTALL_TYPE="reinstall"
fi

DIR_OWN=$WS_USER:$WS_GROUP
CONF_OWN=$WS_USER:$WS_GROUP

chmod $DIR_MOD "$LSWS_HOME"




if [ $INSTALL_TYPE = "upgrade" ]; then
	printf "%s" "Would you like to reset the login password for Administration Web Interface [y/N]? "
	read TMP_URC
	echo ""
	if [ "x$TMP_URC" != "x" ]; then
		if [ `expr "$TMP_URC" : '[Yy]'` -gt 0 ]; then
			SET_LOGIN=1
		fi
	fi
fi		

if [ $SET_LOGIN -eq 1 ]; then

# get admin user name and password

	SUCC=0
	cat <<EOF

Please specify the user name of the administrator.
This is the user name required to log into the administration web interface.

EOF

	printf "%s" "User name [admin]: "
	read ADMIN_USER
	if [ "x$ADMIN_USER" = "x" ]; then
		ADMIN_USER=admin
	fi

	cat <<EOF

Please specify the administrator's password.
This is the password required to log into the administration web interface.

EOF

	while [ $SUCC -eq "0" ];  do
		printf "%s" "Password: "
		stty -echo
		read PASS_ONE
		stty echo
		echo ""
		if [ `expr "$PASS_ONE" : '.*'` -ge 6 ]; then
			printf "%s" "Retype password: "
			stty -echo
			read PASS_TWO
			stty echo
			echo ""
			if [ "x$PASS_ONE" = "x$PASS_TWO" ]; then
				SUCC=1
			else
				echo ""
				echo "[ERROR] Sorry, passwords does not match. Try again!"
				echo ""
			fi
		else
			echo ""
			echo "[ERROR] Sorry, password must be at least 6 characters!"
			echo ""
		fi
	done


# generate password file

	ENCRYPT_PASS=`"$LSINSTALL_DIR/admin/fcgi-bin/admin_php" -q "$LSINSTALL_DIR/admin/misc/htpasswd.php" $PASS_ONE`
	echo "$ADMIN_USER:$ENCRYPT_PASS" > "$LSINSTALL_DIR/admin/conf/htpasswd"

fi

SETUP_PHP=0
ADMIN_PORT=7080

if [ $INSTALL_TYPE = "reinstall" ]; then

if [ $INST_USER = "root" ]; then
	cat <<EOF

As you are the root user, you must choose the user and group
whom the web server will be running as. For security reason, you should choose
a non-system user who does not have login shell and home directory such as
'nobody'.

EOF
# get user name 
	SUCC=0
	while [ $SUCC -eq "0" ]; do
		printf "%s" "User [nobody]: "
		read TMP_USER
		if [ "x$TMP_USER" = "x" ]; then
			TMP_USER=nobody
		fi
		USER_INFO=`id $TMP_USER 2>/dev/null`
		TST_USER=`expr "$USER_INFO" : 'uid=.*(\(.*\)) gid=.*'`
		if [ "x$TST_USER" = "x$TMP_USER" ]; then
			USER_ID=`expr "$USER_INFO" : 'uid=\(.*\)(.*) gid=.*'`
			if [ $USER_ID -gt 50 ]; then  
				WS_USER=$TMP_USER
				SUCC=1
			else
				cat <<EOF

[ERROR] It is not allowed to run LiteSpeed web server on behalf of a 
privileged user, user id must be greater than 50. The user id of user 
'$TMP_USER' is '$USER_ID'.

EOF
			fi
		else
			cat <<EOF 

[ERROR] '$TMP_USER' is not valid user name in your system, please choose 
another user or create user '$TMP_USER' first.

EOF

		fi
	done
fi

# get group name 
SUCC=0
TMP_GROUPS=`groups $WS_USER`
TST_GROUPS=`expr "$TMP_GROUPS" : '.*:\(.*\)'`
if [ "x$TST_GROUPS" = "x" ]; then
    TST_GROUPS=$TMP_GROUPS
fi

D_GROUP=`$ID_GROUPS $WS_USER`
D_GROUP=`expr "$D_GROUP" : '.*gid=[0-9]*(\(.*\)) groups=.*'`
echo "Please choose the group that the web server running as."
echo 
while [ $SUCC -eq "0" ];  do
    echo "User '$WS_USER' is the member of following group(s): $TST_GROUPS"
	printf "%s" "Group [$D_GROUP]: "
	read TMP_GROUP
	if [ "x$TMP_GROUP" = "x" ]; then
		TMP_GROUP=$D_GROUP
	fi
	GRP_RET=`echo $TST_GROUPS | grep -w "$TMP_GROUP"`
	if [ "x$GRP_RET" != "x" ]; then
		WS_GROUP=$TMP_GROUP
		SUCC=1
	else
		cat <<EOF

[ERROR] '$TMP_GROUP' is not valid group for user '$WS_USER', please choose
another group in the list or add user '$WS_USER' to group '$TMP_GROUP' 
first.

EOF
	fi
done

DIR_OWN=$WS_USER:$WS_GROUP
CONF_OWN=$WS_USER:$WS_GROUP

if [ $INST_USER = "root" ]; then
	if [ $SUCC -eq "1" ]; then
		chown -R "$DIR_OWN" "$LSWS_HOME"
	fi
fi


RUNNING_PROCESS=`$PS_CMD | grep lshttpd | grep -v grep`
if [ "x$RUNNING_PROCESS" != "x" ]; then
	cat <<EOF
LiteSpeed web server is running, in order to continue installation, the server 
must be stopped.

EOF
	printf "Would you like to stop it now? [Y/n]"
	read TMP_YN
	echo ""
	if [ "x$TMP_YN" = "x" ] || [ `expr "$TMP_YN" : '[Yy]'` -gt 0 ]; then
		$LSINSTALL_DIR/bin/lswsctrl stop
		RUNNING_PROCESS=`$PS_CMD | grep lshttpd | grep -v grep`
		if [ "x$RUNNING_PROCESS" != "x" ]; then
			echo "Failed to stop server, installation abort!"
			exit 1
		fi
	else
		echo "Installation abort!"
		exit 1
	fi
fi


# get normal TCP port

	cat <<EOF

Please specify the port for normal HTTP service.
Port 80 is the standard HTTP port, only 'root' user is allowed to use 
port 80, if you have another web server running on port 80, you need to
specify another port or stop the other web server before starting LiteSpeed
Web Server.
You can access the normal web page at http://<YOUR_HOST>:<HTTP_PORT>/

EOF

	SUCC=0
	DEFAULT_PORT=8088
	while [ $SUCC -eq "0" ];  do
		printf "%s" "HTTP port [$DEFAULT_PORT]: "
		read TMP_PORT
		if [ "x$TMP_PORT" = "x" ]; then
			TMP_PORT=$DEFAULT_PORT
		fi
		SUCC=1
		if [ `expr "$TMP_PORT" : '.*[^0-9]'` -gt 0 ]; then
			echo "[ERROR] Only digits is allowed, try again!"
			SUCC=0
		fi
		if  [ $SUCC -eq 1 ]; then
			if [ $INST_USER != "root" ]; then
				if [ $TMP_PORT -le 1024 ]; then
					echo "[ERROR] Only 'root' can use port below 1024, try again!"
					SUCC=0
				fi
			fi
		fi
		if [ $SUCC -eq 1 ]; then
			if [ `netstat -an | grep -w $TMP_PORT | grep -w LISTEN | wc -l` -gt 0 ]; then
				echo "[ERROR] Port $TMP_PORT is in use now, stop the server using this port first,"
				echo "        or choose another port."
				SUCC=0
			fi
		fi
	done

	HTTP_PORT=$TMP_PORT

# get administration TCP port

	cat <<EOF

Please specify the HTTP port for the administration web interface,
which can be accessed through http://<YOUR_HOST>:<ADMIN_PORT>/

EOF

	SUCC=0
	DEFAULT_PORT=7080
	while [ $SUCC -eq "0" ];  do
		printf "%s" "Admin HTTP port [$DEFAULT_PORT]: "
		read TMP_PORT
		if [ "x$TMP_PORT" = "x" ]; then
			TMP_PORT=$DEFAULT_PORT
		fi
		SUCC=1
		if [ `expr "$TMP_PORT" : '.*[^0-9]'` -gt 0 ]; then
			echo "[ERROR] Only digits is allowed, try again!"
			SUCC=0
		fi
		if  [ $SUCC -eq 1 ]; then
			if [ $INST_USER != "root" ]; then
				if [ $TMP_PORT -le 1024 ]; then
					echo "[ERROR] Only 'root' can use port below 1024, try again!"
					SUCC=0
				fi
			fi
		fi
		if  [ $SUCC -eq 1 ]; then
			if [ $TMP_PORT -eq $HTTP_PORT ]; then
				echo "[ERROR] The admin HTTP port must be different from the normal HTTP port!"
				SUCC=0
			fi
		fi

		if [ $SUCC -eq 1 ]; then
			if [ `netstat -an | grep -w $TMP_PORT | grep -w LISTEN | wc -l` -gt 0 ]; then
				echo "[ERROR] Port $TMP_PORT is in use, stop the server that using this port first,"
				echo "        or choose another port."
				SUCC=0
			fi
		fi
	done

	ADMIN_PORT=$TMP_PORT


#	ADMIN_PHP_PORT=7201
#	SUCC=0
#	while [ $SUCC -eq "0" ];  do
#		if [ `netstat -an | grep -w $ADMIN_PHP_PORT | grep -w LISTEN | wc -l` -eq 0 ]; then
#			SUCC=1
#		fi
#		ADMIN_PHP_PORT=`expr $ADMIN_PHP_PORT + 1`
#	done

# setup php handler 

	cat <<EOF

You can setup a global script handler for PHP with the pre-built PHP engine
shipped with this package now. The PHP engine runs as Fast CGI which  
outperforms Apache's mod_php. 
You can always replace the pre-built PHP engine with your customized PHP 
engine.

EOF

	SUCC=0
	SETUP_PHP=1
	printf "%s" "Setup up PHP [Y/n]: "
	read TMP_YN
	if [ "x$TMP_YN" != "x" ]; then
		if [ `expr "$TMP_YN" : '[Nn]'` -gt 0 ]; then
			SETUP_PHP=0
		fi
	fi
	if [ $SETUP_PHP -eq 1 ]; then
		PHP_SUFFIX="php"
		printf "%s" "Suffix for PHP script(comma separated list) [$PHP_SUFFIX]: " 
		read TMP_SUFFIX
		if [ "x$TMP_SUFFIX" != "x" ]; then
			PHP_SUFFIX=$TMP_SUFFIX
		fi
#		PHP_PORT=5101
#		SUCC=0
#		while [ $SUCC -eq "0" ];  do
#			if [ `netstat -an | grep -w $PHP_PORT | grep -w LISTEN | wc -l` -eq 0 ]; then
#				SUCC=1
#			fi
#			PHP_PORT=`expr $PHP_PORT + 1`
#		done
	fi
fi



cat <<EOF

Installing, please wait... 

EOF

# generate configuration from template

#sed -e "s/%ADMIN_PORT%/$ADMIN_PORT/" -e "s/%PHP_FCGI_PORT%/$ADMIN_PHP_PORT/" "$LSINSTALL_DIR/admin/conf/admin_config.xml.in" > "$LSINSTALL_DIR/admin/conf/admin_config.xml"

sed -e "s/%ADMIN_PORT%/$ADMIN_PORT/" "$LSINSTALL_DIR/admin/conf/admin_config.xml.in" > "$LSINSTALL_DIR/admin/conf/admin_config.xml"

sed -e "s/%USER%/$WS_USER/" -e "s/%GROUP%/$WS_GROUP/" -e "s/%HTTP_PORT%/$HTTP_PORT/" "$LSINSTALL_DIR/conf/httpd_config.xml.in" > "$LSINSTALL_DIR/conf/httpd_config.xml.tmp"

if [ $SETUP_PHP -eq 1 ]; then
	sed -e "s/%PHP_BEGIN%//" -e "s/%PHP_END%//" -e "s/%PHP_SUFFIX%/$PHP_SUFFIX/" -e "s/%PHP_PORT%/$PHP_PORT/" "$LSINSTALL_DIR/conf/httpd_config.xml.tmp" > "$LSINSTALL_DIR/conf/httpd_config.xml"
else
	sed -e "s/%PHP_BEGIN%/<!--/" -e "s/%PHP_END%/-->/" -e "s/%PHP_SUFFIX%/php/" -e "s/%PHP_PORT%/5201/" "$LSINSTALL_DIR/conf/httpd_config.xml.tmp" > "$LSINSTALL_DIR/conf/httpd_config.xml"
fi

sed "s:%LSWS_CTRL%:$LSWS_HOME/bin/lswsctrl:" "$LSINSTALL_DIR/admin/misc/lsws.rc.in" > "$LSINSTALL_DIR/admin/misc/lsws.rc"


"$INST_UTIL" MKDIR "$DIR_OWN" $DIR_MOD docs DEFAULT logs  
"$INST_UTIL" MKDIR "$DIR_OWN" $SDIR_MOD admin bin conf fcgi-bin lib

"$INST_UTIL" MKDIR "$DIR_OWN" $SDIR_MOD admin/conf admin/fcgi-bin admin/html admin/logs admin/misc
"$INST_UTIL" MKDIR "$DIR_OWN" $SDIR_MOD conf/cert
"$INST_UTIL" CPFILE "$DIR_OWN" $EXEC_MOD admin/fcgi-bin/admin_php 
"$INST_UTIL" CPFILE "$DIR_OWN" $EXEC_MOD admin/misc/rc-inst.sh admin/misc/admpass.sh admin/misc/rc-uninst.sh admin/misc/uninstall.sh admin/misc/lsws.rc admin/misc/enable_phpa.sh
"$INST_UTIL" CPFILE "$DIR_OWN" $DOC_MOD admin/misc/gdb-bt admin/misc/htpasswd.php admin/misc/php.ini

if [ $SET_LOGIN -eq 1 ]; then
	"$INST_UTIL" CPFILE "$CONF_OWN" $CONF_MOD admin/conf/htpasswd
else
	"$INST_UTIL" CCPFILE "$CONF_OWN" $CONF_MOD admin/conf/htpasswd
fi
if [ $INSTALL_TYPE = "upgrade" ]; then
	"$INST_UTIL" CCPFILE "$CONF_OWN" $CONF_MOD admin/conf/admin_config.xml
	"$INST_UTIL" CCPFILE "$CONF_OWN" $CONF_MOD admin/conf/php.ini
	"$INST_UTIL" CCPFILE "$CONF_OWN" $CONF_MOD conf/httpd_config.xml conf/mime.properties
	mv -f "$LSWS_HOME/bin/lshttpd" "$LSWS_HOME/bin/lshttpd.old"
else
	"$INST_UTIL" CPFILE "$CONF_OWN" $CONF_MOD admin/conf/admin_config.xml
	"$INST_UTIL" CPFILE "$CONF_OWN" $CONF_MOD admin/conf/php.ini
	"$INST_UTIL" CPFILE "$CONF_OWN" $CONF_MOD conf/httpd_config.xml conf/mime.properties
fi
if [ $SETUP_PHP -eq 1 ]; then
	if [ -f "$LSWS_HOME/fcgi-bin/php" ]; then
		mv -f "$LSWS_HOME/fcgi-bin/php" "$LSWS_HOME/fcgi-bin/php.old"
		echo "Your current PHP engine $LSWS_HOME/fcgi-bin/php is renamed to php.old"
	fi
	if [ ! -f "$LSWS_HOME/conf/php.ini" ]; then
		cp -f "$LSWS_HOME/admin/conf/php.ini" "$LSWS_HOME/conf/php.ini"
	    chown "$CONF_OWN" "$LSWS_HOME/conf/php.ini"
		chmod "$CONF_MOD" "$LSWS_HOME/conf/php.ini"
	fi
	cp -f "$LSWS_HOME/admin/fcgi-bin/admin_php" "$LSWS_HOME/fcgi-bin/php"
    chown "$DIR_OWN" "$LSWS_HOME/fcgi-bin/php"
    chmod "$EXEC_MOD" "$LSWS_HOME/fcgi-bin/php"
fi
"$INST_UTIL" MKDIR "$DIR_OWN" $DIR_MOD DEFAULT/conf DEFAULT/html DEFAULT/logs DEFAULT/cgi-bin DEFAULT/fcgi-bin
"$INST_UTIL" CPDIR "$DIR_OWN" $DOC_MOD DEFAULT/conf DEFAULT/html DEFAULT/cgi-bin
"$INST_UTIL" CPDIR "$DIR_OWN" $DOC_MOD admin/html
chmod -R o+r "$LSWS_HOME/admin/html"
chmod -R o+r "$LSWS_HOME/DEFAULT"


"$INST_UTIL" CPFILE "$DIR_OWN" $EXEC_MOD bin/lswsctrl bin/lshttpd bin/lscgid
"$INST_UTIL" CPFILE "$DIR_OWN" $DOC_MOD docs/*.html docs/*.css docs/*.gif lib/apc.so

cat <<EOF

In order to maximize the performance of PHP, a pre-built PHP opcode cache
can be installed automatically. An opcode cache increases performance of
PHP scripts by caching them in compiled state, the overhead of compiling
is voided.

Note: If an opcode cache has been installed already, you do not need to
      change it. If you built PHP binary by yourself, you need to built
      PHP opcode cache from source as well, unless the version of your
      PHP binary is same as the pre-built PHP opcode cache built for.

EOF

printf "%s" "Would you like to change PHP opcode cache setting [y/N]? "

read PHPACC
echo

if [ "x$PHPACC" = "x" ]; then
	PHPACC=n
fi
if [ `expr "$PHPACC" : '[Yy]'` -gt 0 ]; then
	$LSWS_HOME/admin/misc/enable_phpa.sh
fi

cat <<EOF
Congratulations! The LiteSpeed Web Server has been successfully installed.
Command line script - "$LSWS_HOME/bin/lswsctrl"
can be used to start or stop the server.

It is recommended to limit access to the web administration interface. 
Right now the interface can be accessed from anywhere where this
machine can be reached over the network. Three options are available:
1. If the interface needs to be accessed only from this machine, just 
   change the listener for the interface to only listening on the loopback 
   interface - localhost(127.0.0.1). 
2. If the interface needs to be accessible from limited IP addresses or sub 
   networks, then set up access control rules for the interface accordingly.
3. If the interface has to be accessible via internet, SSL (Secure Sockets 
   Layer) should be used. Please read respective HOW-TOs on SSL configuration.

To change configurations of the interface, login and click 
"Interface Configuration" button on the main page.
The administration interface is located at http://localhost:<ADMIN_PORT>/
or http://<ip_or_Hostname_of_this_machine>:<ADMIN_PORT>/ 

EOF

if [ $INST_USER = "root" ]; then
	if [ $INSTALL_TYPE != "upgrade" ]; then
		printf "%s" "Would you like to have LiteSpeed Web Server started automatically when the machine restarts [Y/n]? "
		read START_SERVER
		echo 

		if [ "x$START_SERVER" = "x" ]; then
			START_SERVER=y
		fi
		if [ `expr "$START_SERVER" : '[Yy]'` -gt 0 ]; then
			$LSWS_HOME/admin/misc/rc-inst.sh
		else
			cat <<EOF
If you want to start the web server automatically later, just run 
	"$LSWS_HOME/admin/misc/rc-inst.sh"
to install the service control script.

EOF
		fi
	fi
fi

if [ $INSTALL_TYPE != "upgrade" ]; then
	printf "%s" "Would you like to start it right now [Y/n]? "
else
	printf "%s" "Would you like to restart it right now [Y/n]? "
fi
read START_SERVER
echo 

if [ "x$START_SERVER" = "x" ]; then
	START_SERVER=y
fi

if [ `expr "$START_SERVER" : '[Yy]'` -gt 0 ]; then
	if [ $INSTALL_TYPE != "upgrade" ]; then
		"$LSWS_HOME/bin/lswsctrl" start
	else
		"$LSWS_HOME/bin/lswsctrl" restart
	fi
else
	exit 0
fi

sleep 1
RUNNING_PROCESS=`$PS_CMD | grep lshttpd | grep -v grep`

if [ "x$RUNNING_PROCESS" != "x" ]; then

	cat <<EOF

LiteSpeed Web Server started successfully! Have fun!

EOF
	exit 0
else

	cat <<EOF

[ERROR] Failed to start the web server. For trouble shooting information,
        please refer to documents in "$LSWS_HOME/docs/".

EOF
fi


